# -*- coding: utf-8 -*-
"""This file contains a helper library to read binary files."""

import binascii
import logging
import os

from plaso.lib import py2to3


def ByteArrayCopyToString(byte_array, codepage=u'utf-8'):
  """Copies a UTF-8 encoded byte array into a Unicode string.

  Args:
    byte_array: A byte array containing an UTF-8 encoded string.
    codepage: The codepage of the byte stream.

  Returns:
    A Unicode string.
  """
  byte_stream = b''.join(map(chr, byte_array))
  return ByteStreamCopyToString(byte_stream, codepage=codepage)


def ByteStreamCopyToString(byte_stream, codepage=u'utf-8'):
  """Copies a UTF-8 encoded byte stream into a Unicode string.

  Args:
    byte_stream: A byte stream containing an UTF-8 encoded string.
    codepage: The codepage of the byte stream.

  Returns:
    A Unicode string.
  """
  try:
    string = byte_stream.decode(codepage)
  except UnicodeDecodeError:
    logging.warning(
        u'Unable to decode {0:s} formatted byte stream.'.format(codepage))
    string = byte_stream.decode(codepage, errors='ignore')

  string, _, _ = string.partition(u'\x00')
  return string


def ByteStreamCopyToUTF16Stream(byte_stream, byte_stream_size=None):
  """Reads an UTF-16 formatted stream from a byte stream.

  The UTF-16 formatted stream should be terminated by an end-of-string
  character (\x00\x00). Otherwise the function reads up to the byte stream size.

  Args:
    byte_stream: The byte stream that contains the UTF-16 formatted stream.
    byte_stream_size: Optional byte stream size or None if the entire
                      byte stream should be read.

  Returns:
    String containing the UTF-16 formatted stream.
  """
  byte_stream_index = 0
  if not byte_stream_size:
    byte_stream_size = len(byte_stream)

  while byte_stream_index + 1 < byte_stream_size:
    if (byte_stream[byte_stream_index] == b'\x00' and
        byte_stream[byte_stream_index + 1] == b'\x00'):
      break

    byte_stream_index += 2

  return byte_stream[0:byte_stream_index]


def ReadUTF16Stream(file_object, offset=None, byte_size=0):
  """Reads an UTF-16 formatted stream from a file-like object.

  Reads an UTF-16 formatted stream that's terminated by
  an end-of-string character (\x00\x00) or up to the byte size.

  Args:
    file_object: A file-like object to read the data from.
    offset: An offset into the file object data, if -1 or not set
            the current location into the file object data is used.
    byte_size: Maximum number of bytes to read or 0 if the function
               should keep reading up to the end of file.

  Returns:
    An Unicode string.
  """
  if offset is not None:
    file_object.seek(offset, os.SEEK_SET)

  char_buffer = []

  stream_index = 0
  char_raw = file_object.read(2)
  while char_raw:
    if byte_size and stream_index >= byte_size:
      break

    if b'\x00\x00' in char_raw:
      break
    char_buffer.append(char_raw)
    stream_index += 2
    char_raw = file_object.read(2)

  return ReadUTF16(b''.join(char_buffer))


def UTF16StreamCopyToString(byte_stream, byte_stream_size=None):
  """Copies an UTF-16 formatted byte stream to a string.

  The UTF-16 formatted byte stream should be terminated by an end-of-string
  character (\x00\x00). Otherwise the function reads up to the byte stream size.

  Args:
    byte_stream: The UTF-16 formatted byte stream.
    byte_stream_size: The byte stream size or None if the entire byte stream
                      should be used.

  Returns:
    An Unicode string.
  """
  utf16_stream = ByteStreamCopyToUTF16Stream(
      byte_stream, byte_stream_size=byte_stream_size)

  try:
    return utf16_stream.decode(u'utf-16-le')
  except (UnicodeDecodeError, UnicodeEncodeError) as exception:
    logging.error(u'Unable to decode string: {0:s} with error: {1:s}'.format(
        HexifyBuffer(utf16_stream), exception))

  return utf16_stream.decode(u'utf-16-le', errors=u'ignore')


def ArrayOfUTF16StreamCopyToString(byte_stream, byte_stream_size=None):
  """Copies an array of UTF-16 formatted byte streams to an array of strings.

  The UTF-16 formatted byte stream should be terminated by an end-of-string
  character (\x00\x00). Otherwise the function reads up to the byte stream size.

  Args:
    byte_stream: The UTF-16 formatted byte stream.
    byte_stream_size: The byte stream size or None if the entire byte stream
                      should be used.

  Returns:
    An array of Unicode strings.
  """
  array_of_strings = []
  utf16_stream_start = 0
  byte_stream_index = 0
  if not byte_stream_size:
    byte_stream_size = len(byte_stream)

  while byte_stream_index + 1 < byte_stream_size:
    if (byte_stream[byte_stream_index] == b'\x00' and
        byte_stream[byte_stream_index + 1] == b'\x00'):

      if byte_stream_index - utf16_stream_start <= 2:
        break

      array_of_strings.append(
          byte_stream[utf16_stream_start:byte_stream_index].decode(
              u'utf-16-le'))
      utf16_stream_start = byte_stream_index + 2

    byte_stream_index += 2

  return array_of_strings


def ArrayOfUTF16StreamCopyToStringTable(byte_stream, byte_stream_size=None):
  """Copies an array of UTF-16 formatted byte streams to a string table.

  The string table is a dict of strings with the byte offset as their key.
  The UTF-16 formatted byte stream should be terminated by an end-of-string
  character (\x00\x00). Otherwise the function reads up to the byte stream size.

  Args:
    byte_stream: The UTF-16 formatted byte stream.
    byte_stream_size: The byte stream size or None if the entire byte stream
                      should be used.

  Returns:
    A dict of Unicode strings with the byte offset as their key.
  """
  string_table = {}
  utf16_stream_start = 0
  byte_stream_index = 0
  if not byte_stream_size:
    byte_stream_size = len(byte_stream)

  while byte_stream_index + 1 < byte_stream_size:
    if (byte_stream[byte_stream_index] == b'\x00' and
        byte_stream[byte_stream_index + 1] == b'\x00'):

      if byte_stream_index - utf16_stream_start <= 2:
        break

      string = byte_stream[utf16_stream_start:byte_stream_index].decode(
          u'utf-16-le')
      string_table[utf16_stream_start] = string
      utf16_stream_start = byte_stream_index + 2

    byte_stream_index += 2

  return string_table


def ReadUTF16(string_buffer):
  """Returns a decoded UTF-16 string from a string buffer."""
  if isinstance(string_buffer, (list, tuple)):
    use_buffer = u''.join(string_buffer)
  else:
    use_buffer = string_buffer

  if not isinstance(use_buffer, py2to3.STRING_TYPES):
    return u''

  try:
    return use_buffer.decode(u'utf-16').replace(u'\x00', u'')
  except SyntaxError as exception:
    logging.error(u'Unable to decode string: {0:s} with error: {1:s}.'.format(
        HexifyBuffer(string_buffer), exception))
  except (UnicodeDecodeError, UnicodeEncodeError) as exception:
    logging.error(u'Unable to decode string: {0:s} with error: {1:s}'.format(
        HexifyBuffer(string_buffer), exception))

  return use_buffer.decode(u'utf-16', errors=u'ignore').replace(u'\x00', u'')


def HexifyBuffer(string_buffer):
  """Return a string with the hex representation of a string buffer."""
  chars = []
  for char in string_buffer:
    chars.append(binascii.hexlify(char))

  return u'\\x{0:s}'.format(u'\\x'.join(chars))



#!/usr/bin/python
# -*- coding: utf-8 -*-
"""Tests for the shell item event attribute container."""

import unittest

from plaso.containers import shell_item_events

from tests import test_lib as shared_test_lib


class ShellItemFileEntryEventDataTest(shared_test_lib.BaseTestCase):
  """Tests for the shell item event attribute container."""

  def testGetAttributeNames(self):
    """Tests the GetAttributeNames function."""
    attribute_container = shell_item_events.ShellItemFileEntryEventData()

    expected_attribute_names = [
        u'data_type', u'file_reference', u'localized_name', u'long_name',
        u'name', u'offset', u'origin', u'query', u'shell_item_path']

    attribute_names = sorted(attribute_container.GetAttributeNames())

    self.assertEqual(attribute_names, expected_attribute_names)


if __name__ == '__main__':
  unittest.main()



#!/usr/bin/python
# -*- coding: utf-8 -*-
"""Tests for the extractor classes."""

import os
import shutil
import unittest

from dfvfs.helpers import file_system_searcher
from dfvfs.lib import definitions as dfvfs_definitions
from dfvfs.path import factory as path_spec_factory
from dfvfs.resolver import context

from plaso.engine import extractors

from tests import test_lib as shared_test_lib


# TODO: add EventExtractorTest


class PathSpecExtractorTest(shared_test_lib.BaseTestCase):
  """Tests for the path specification extractor."""

  def _GetFilePaths(self, path_specs):
    """Retrieves file paths from path specifications.

    Args:
      path_specs (list[dfvfs.PathSpec]): path specifications.

    Returns:
      list[str]: file paths.
    """
    file_paths = []
    for path_spec in path_specs:
      data_stream = getattr(path_spec, u'data_stream', None)
      location = getattr(path_spec, u'location', None)
      if location is not None:
        if data_stream:
          location = u'{0:s}:{1:s}'.format(location, data_stream)
        file_paths.append(location)

    return file_paths

  def _GetFindSpecs(self, location_expressions):
    """Retrieves find specifications from location expressions.

    Args:
      location_expressions (list[str]): location regular expressions.

    Returns:
      list[dfvfs.FindSpec]: find specifications for the file system searcher.
    """
    find_specs = []
    for location_expression in location_expressions:
      # Convert the filter paths into a list of path segments and strip
      # the root path segment.
      path_segments = location_expression.split(u'/')
      path_segments.pop(0)

      find_spec = file_system_searcher.FindSpec(
          location_regex=path_segments, case_sensitive=False)
      find_specs.append(find_spec)

    return find_specs

  @shared_test_lib.skipUnlessHasTestFile([u'syslog.bz2'])
  @shared_test_lib.skipUnlessHasTestFile([u'syslog.tgz'])
  @shared_test_lib.skipUnlessHasTestFile([u'syslog.zip'])
  @shared_test_lib.skipUnlessHasTestFile([u'wtmp.1'])
  def testExtractPathSpecsFileSystem(self):
    """Tests the ExtractPathSpecs function on the file system."""
    test_files = [
        self._GetTestFilePath([u'syslog.bz2']),
        self._GetTestFilePath([u'syslog.tgz']),
        self._GetTestFilePath([u'syslog.zip']),
        self._GetTestFilePath([u'wtmp.1'])]

    with shared_test_lib.TempDirectory() as temp_directory:
      for a_file in test_files:
        shutil.copy(a_file, temp_directory)

      source_path_spec = path_spec_factory.Factory.NewPathSpec(
          dfvfs_definitions.TYPE_INDICATOR_OS, location=temp_directory)

      resolver_context = context.Context()
      test_extractor = extractors.PathSpecExtractor()
      path_specs = list(test_extractor.ExtractPathSpecs(
          [source_path_spec], resolver_context=resolver_context))

      self.assertEqual(len(path_specs), 4)

  @shared_test_lib.skipUnlessHasTestFile([u'System.evtx'])
  @shared_test_lib.skipUnlessHasTestFile([u'testdir', u'filter_1.txt'])
  @shared_test_lib.skipUnlessHasTestFile([u'testdir', u'filter_3.txt'])
  def testExtractPathSpecsFileSystemWithFindSpecs(self):
    """Tests the ExtractPathSpecs function with find specifications."""
    location_expressions = [
        u'/test_data/testdir/filter_.+.txt',
        u'/test_data/.+evtx',
        u'/AUTHORS',
        u'/does_not_exist/some_file_[0-9]+txt']

    source_path_spec = path_spec_factory.Factory.NewPathSpec(
        dfvfs_definitions.TYPE_INDICATOR_OS, location=u'.')

    resolver_context = context.Context()
    test_extractor = extractors.PathSpecExtractor()

    find_specs = self._GetFindSpecs(location_expressions)
    path_specs = list(test_extractor.ExtractPathSpecs(
        [source_path_spec], find_specs=find_specs,
        resolver_context=resolver_context))

    # Two files with test_data/testdir/filter_*.txt, AUTHORS
    # and test_data/System.evtx and test_data/System2.evtx.
    self.assertEqual(len(path_specs), 5)

    paths = self._GetFilePaths(path_specs)

    current_directory = os.getcwd()

    expected_path = os.path.join(
        current_directory, u'test_data', u'testdir', u'filter_1.txt')
    self.assertTrue(expected_path in paths)

    expected_path = os.path.join(
        current_directory, u'test_data', u'testdir', u'filter_2.txt')
    self.assertFalse(expected_path in paths)

    expected_path = os.path.join(
        current_directory, u'test_data', u'testdir', u'filter_3.txt')
    self.assertTrue(expected_path in paths)

    expected_path = os.path.join(
        current_directory, u'AUTHORS')
    self.assertTrue(expected_path in paths)

  @shared_test_lib.skipUnlessHasTestFile([u'syslog_image.dd'])
  def testExtractPathSpecsStorageMediaImage(self):
    """Tests the ExtractPathSpecs function an image file.

    The image file contains the following files:
    * logs/hidden.zip
    * logs/sys.tgz

    The hidden.zip file contains one file, syslog, which is the
    same for sys.tgz.

    The end results should therefore be:
    * logs/hidden.zip (unchanged)
    * logs/hidden.zip:syslog (the text file extracted out)
    * logs/sys.tgz (unchanged)
    * logs/sys.tgz (read as a GZIP file, so not compressed)
    * logs/sys.tgz:syslog.gz (A GZIP file from the TAR container)
    * logs/sys.tgz:syslog.gz:syslog (the extracted syslog file)

    This means that the collection script should collect 6 files in total.
    """
    test_file = self._GetTestFilePath([u'syslog_image.dd'])

    volume_path_spec = path_spec_factory.Factory.NewPathSpec(
        dfvfs_definitions.TYPE_INDICATOR_OS, location=test_file)
    source_path_spec = path_spec_factory.Factory.NewPathSpec(
        dfvfs_definitions.TYPE_INDICATOR_TSK, location=u'/',
        parent=volume_path_spec)

    resolver_context = context.Context()
    test_extractor = extractors.PathSpecExtractor()
    path_specs = list(test_extractor.ExtractPathSpecs(
        [source_path_spec], resolver_context=resolver_context))

    self.assertEqual(len(path_specs), 3)

  @shared_test_lib.skipUnlessHasTestFile([u'ímynd.dd'])
  def testExtractPathSpecsStorageMediaImageWithFilter(self):
    """Tests the ExtractPathSpecs function on an image file with a filter."""
    location_expressions = [
        u'/a_directory/.+zip',
        u'/a_directory/another.+',
        u'/passwords.txt']

    test_file = self._GetTestFilePath([u'ímynd.dd'])

    volume_path_spec = path_spec_factory.Factory.NewPathSpec(
        dfvfs_definitions.TYPE_INDICATOR_OS, location=test_file)
    source_path_spec = path_spec_factory.Factory.NewPathSpec(
        dfvfs_definitions.TYPE_INDICATOR_TSK, location=u'/',
        parent=volume_path_spec)

    resolver_context = context.Context()
    test_extractor = extractors.PathSpecExtractor()

    find_specs = self._GetFindSpecs(location_expressions)
    path_specs = list(test_extractor.ExtractPathSpecs(
        [source_path_spec], find_specs=find_specs,
        resolver_context=resolver_context))

    self.assertEqual(len(path_specs), 2)

    paths = self._GetFilePaths(path_specs)

    # path_specs[0]
    # type: TSK
    # file_path: '/a_directory/another_file'
    # container_path: 'test_data/ímynd.dd'
    # image_offset: 0
    self.assertEqual(paths[0], u'/a_directory/another_file')

    # path_specs[1]
    # type: TSK
    # file_path: '/passwords.txt'
    # container_path: 'test_data/ímynd.dd'
    # image_offset: 0
    self.assertEqual(paths[1], u'/passwords.txt')

  @shared_test_lib.skipUnlessHasTestFile([u'multi_partition_image.vmdk'])
  def testExtractPathSpecsStorageMediaImageWithPartitions(self):
    """Tests the ExtractPathSpecs function an image file with partitions.

    The image file contains 2 partitions, p1 and p2, both with a NFTS
    file systems.
    """
    # Note that the source file is a RAW (VMDK flat) image.
    test_file = self._GetTestFilePath([u'multi_partition_image.vmdk'])

    image_path_spec = path_spec_factory.Factory.NewPathSpec(
        dfvfs_definitions.TYPE_INDICATOR_OS, location=test_file)

    p1_path_spec = path_spec_factory.Factory.NewPathSpec(
        dfvfs_definitions.TYPE_INDICATOR_TSK_PARTITION, location=u'/p1',
        part_index=2, start_offset=0x00010000, parent=image_path_spec)
    p1_file_system_path_spec = path_spec_factory.Factory.NewPathSpec(
        dfvfs_definitions.TYPE_INDICATOR_TSK, location=u'/',
        parent=p1_path_spec)

    p2_path_spec = path_spec_factory.Factory.NewPathSpec(
        dfvfs_definitions.TYPE_INDICATOR_TSK_PARTITION, location=u'/p2',
        part_index=3, start_offset=0x00510000, parent=image_path_spec)
    p2_file_system_path_spec = path_spec_factory.Factory.NewPathSpec(
        dfvfs_definitions.TYPE_INDICATOR_TSK, location=u'/',
        parent=p2_path_spec)

    resolver_context = context.Context()
    test_extractor = extractors.PathSpecExtractor(resolver_context)

    path_specs = list(test_extractor.ExtractPathSpecs(
        [p1_file_system_path_spec, p2_file_system_path_spec],
        resolver_context=resolver_context))

    expected_paths_p1 = [
        u'/$AttrDef',
        u'/$BadClus',
        u'/$BadClus:$Bad',
        u'/$Bitmap',
        u'/$Boot',
        u'/$Extend',
        u'/$Extend/$ObjId',
        u'/$Extend/$Quota',
        u'/$Extend/$Reparse',
        u'/$Extend/$RmMetadata',
        u'/$Extend/$RmMetadata/$Repair',
        u'/$Extend/$RmMetadata/$Repair:$Config',
        u'/$Extend/$RmMetadata/$TxfLog',
        u'/$LogFile',
        u'/$MFT',
        u'/$MFTMirr',
        u'/$Secure',
        u'/$Secure:$SDS',
        u'/$UpCase',
        u'/$Volume',
        u'/file1.txt',
        u'/file2.txt']

    expected_paths_p2 = [
        u'/$AttrDef',
        u'/$BadClus',
        u'/$BadClus:$Bad',
        u'/$Bitmap',
        u'/$Boot',
        u'/$Extend',
        u'/$Extend/$ObjId',
        u'/$Extend/$Quota',
        u'/$Extend/$Reparse',
        u'/$Extend/$RmMetadata',
        u'/$Extend/$RmMetadata/$Repair',
        u'/$Extend/$RmMetadata/$Repair:$Config',
        u'/$Extend/$RmMetadata/$TxfLog',
        u'/$LogFile',
        u'/$MFT',
        u'/$MFTMirr',
        u'/$Secure',
        u'/$Secure:$SDS',
        u'/$UpCase',
        u'/$Volume',
        u'/file1_on_part_2.txt',
        u'/file2_on_part_2.txt']

    paths = self._GetFilePaths(path_specs)
    expected_paths = expected_paths_p1
    expected_paths.extend(expected_paths_p2)

    self.assertEqual(len(path_specs), len(expected_paths))
    self.assertEqual(sorted(paths), sorted(expected_paths))


if __name__ == '__main__':
  unittest.main()



# -*- coding: utf-8 -*-
"""This file contains the attribute container manager class."""


class AttributeContainersManager(object):
  """Class that implements the attribute container manager."""

  _attribute_container_classes = {}

  @classmethod
  def DeregisterAttributeContainer(cls, attribute_container_class):
    """Deregisters an attribute container class.

    The attribute container classes are identified based on their lower case
    containter type.

    Args:
      attribute_container_class (type): attribute container class.

    Raises:
      KeyError: if attribute container class is not set for
                the corresponding container type.
    """
    container_type = attribute_container_class.CONTAINER_TYPE.lower()
    if container_type not in cls._attribute_container_classes:
      raise KeyError(
          u'Attribute container class not set for container type: '
          u'{0:s}.'.format(attribute_container_class.CONTAINER_TYPE))

    del cls._attribute_container_classes[container_type]

  @classmethod
  def GetAttributeContainer(cls, container_type):
    """Retrieves the attribute container for a specific container type.

    Args:
      container_type (str): container type.

    Returns:
      AttributeContainer: attribute container.
    """
    return cls._attribute_container_classes.get(container_type, None)

  @classmethod
  def RegisterAttributeContainer(cls, attribute_container_class):
    """Registers a attribute container class.

    The attribute container classes are identified based on their lower case
    container type.

    Args:
      attribute_container_class (type): attribute container class.

    Raises:
      KeyError: if attribute container class is already set for
                the corresponding container type.
    """
    container_type = attribute_container_class.CONTAINER_TYPE.lower()
    if container_type in cls._attribute_container_classes:
      raise KeyError((
          u'Attribute container class already set for container type: '
          u'{0:s}.').format(attribute_container_class.CONTAINER_TYPE))

    cls._attribute_container_classes[container_type] = attribute_container_class

  @classmethod
  def RegisterAttributeContainers(cls, attribute_container_classes):
    """Registers attribute container classes.

    The attribute container classes are identified based on their lower case
    container type.

    Args:
      attribute_container_classes (list[type]): attribute container classes.

    Raises:
      KeyError: if attribute container class is already set for
                the corresponding container type.
    """
    for attribute_container_class in attribute_container_classes:
      cls.RegisterAttributeContainer(attribute_container_class)



# -*- coding: utf-8 -*-
"""The UTMPX binary file event formatter."""

from plaso.formatters import interface
from plaso.formatters import manager
from plaso.lib import errors


class UtmpxSessionFormatter(interface.ConditionalEventFormatter):
  """Formatter for an UTMPX session event."""

  DATA_TYPE = u'mac:utmpx:event'

  FORMAT_STRING_PIECES = [
      u'User: {user}',
      u'Status: {status}',
      u'Computer Name: {computer_name}',
      u'Terminal: {terminal}']

  FORMAT_STRING_SHORT_PIECES = [u'User: {user}']

  SOURCE_LONG = u'UTMPX session'
  SOURCE_SHORT = u'LOG'

  # 9, 10 and 11 are only for Darwin and IOS.
  _STATUS_TYPES = {
      0: u'EMPTY',
      1: u'RUN_LVL',
      2: u'BOOT_TIME',
      3: u'OLD_TIME',
      4: u'NEW_TIME',
      5: u'INIT_PROCESS',
      6: u'LOGIN_PROCESS',
      7: u'USER_PROCESS',
      8: u'DEAD_PROCESS',
      9: u'ACCOUNTING',
      10: u'SIGNATURE',
      11: u'SHUTDOWN_TIME'}

  def GetMessages(self, unused_formatter_mediator, event):
    """Determines the formatted message strings for an event object.

    Args:
      formatter_mediator (FormatterMediator): mediates the interactions between
          formatters and other components, such as storage and Windows EventLog
          resources.
      event (EventObject): event.

    Returns:
      tuple(str, str): formatted message string and short message string.

    Raises:
      WrongFormatter: if the event object cannot be formatted by the formatter.
    """
    if self.DATA_TYPE != event.data_type:
      raise errors.WrongFormatter(u'Unsupported data type: {0:s}.'.format(
          event.data_type))

    event_values = event.CopyToDict()

    status_type = event_values.get(u'status_type', None)
    if status_type is not None:
      event_values[u'status'] = self._STATUS_TYPES.get(
          status_type, u'{0:d}'.format(status_type))
    else:
      event_values[u'status'] = u'N/A'

    return self._ConditionalFormatMessages(event_values)


manager.FormattersManager.RegisterFormatter(UtmpxSessionFormatter)



#!/usr/bin/python
# -*- coding: utf-8 -*-
"""Tests for the LS Quarantine database plugin."""

import unittest

from plaso.formatters import ls_quarantine  # pylint: disable=unused-import
from plaso.lib import timelib
from plaso.parsers.sqlite_plugins import ls_quarantine

from tests import test_lib as shared_test_lib
from tests.parsers.sqlite_plugins import test_lib


class LSQuarantinePluginTest(test_lib.SQLitePluginTestCase):
  """Tests for the LS Quarantine database plugin."""

  @shared_test_lib.skipUnlessHasTestFile([u'quarantine.db'])
  def testProcess(self):
    """Tests the Process function on a LS Quarantine database file."""
    plugin = ls_quarantine.LsQuarantinePlugin()
    storage_writer = self._ParseDatabaseFileWithPlugin(
        [u'quarantine.db'], plugin)

    # The quarantine database contains 14 events.
    self.assertEqual(storage_writer.number_of_events, 14)

    events = list(storage_writer.GetEvents())

    # Examine a VLC event.
    event = events[3]

    expected_timestamp = timelib.Timestamp.CopyFromString(
        u'2013-07-08 21:12:03')
    self.assertEqual(event.timestamp, expected_timestamp)

    self.assertEqual(event.agent, u'Google Chrome')
    expected_url = (
        u'http://download.cnet.com/VLC-Media-Player/3001-2139_4-10210434.html'
        u'?spi=40ab24d3c71594a5017d74be3b0c946c')
    self.assertEqual(event.url, expected_url)

    self.assertTrue(u'vlc-2.0.7-intel64.dmg' in event.data)

    # Examine a MacKeeper event.
    event = events[9]

    expected_timestamp = timelib.Timestamp.CopyFromString(
        u'2013-07-12 19:28:58')
    self.assertEqual(event.timestamp, expected_timestamp)

    # Examine a SpeedTest event.
    event = events[10]

    expected_timestamp = timelib.Timestamp.CopyFromString(
        u'2013-07-12 19:30:16')
    self.assertEqual(event.timestamp, expected_timestamp)

    expected_message = (
        u'[Google Chrome] Downloaded: http://mackeeperapp.zeobit.com/aff/'
        u'speedtest.net.6/download.php?affid=460245286&trt=5&utm_campaign='
        u'3ES&tid_ext=P107fSKcSfqpMbcP3sI4fhKmeMchEB3dkAGpX4YIsvM;US;L;1 '
        u'<http://download.mackeeper.zeobit.com/package.php?'
        u'key=460245286&trt=5&landpr=Speedtest>')
    expected_short_message = (
        u'http://mackeeperapp.zeobit.com/aff/speedtest.net.6/download.php?'
        u'affid=4602452...')

    self._TestGetMessageStrings(event, expected_message, expected_short_message)


if __name__ == '__main__':
  unittest.main()



# -*- coding: utf-8 -*-
"""This file contains a parser for the Mac OS X application usage.

The application usage is stored in SQLite database files named
/var/db/application_usage.sqlite
"""

from dfdatetime import posix_time as dfdatetime_posix_time

from plaso.containers import events
from plaso.containers import time_events
from plaso.parsers import sqlite
from plaso.parsers.sqlite_plugins import interface


class MacOSXApplicationUsageEventData(events.EventData):
  """Mac OS X application usage event data.

  Attributes:
    application (str): name of the application.
    app_version (str): version of the application.
    bundle_id (str): bundle identifier of the application.
    count (int): TODO: number of times what?
  """

  DATA_TYPE = u'macosx:application_usage'

  def __init__(self):
    """Initializes event data."""
    super(MacOSXApplicationUsageEventData, self).__init__(
        data_type=self.DATA_TYPE)
    self.application = None
    self.app_version = None
    self.bundle_id = None
    self.count = None


class ApplicationUsagePlugin(interface.SQLitePlugin):
  """Parse Application Usage history files.

  Application usage is a SQLite database that logs down entries
  triggered by NSWorkspaceWillLaunchApplicationNotification and
  NSWorkspaceDidTerminateApplicationNotification NSWorkspace notifications by
  crankd.

  See the code here:
  http://code.google.com/p/google-macops/source/browse/trunk/crankd/\
      ApplicationUsage.py

  Default installation: /var/db/application_usage.sqlite
  """

  NAME = u'appusage'
  DESCRIPTION = u'Parser for Mac OS X application usage SQLite database files.'

  # Define the needed queries.
  QUERIES = [(
      (u'SELECT last_time, event, bundle_id, app_version, app_path, '
       u'number_times FROM application_usage ORDER BY last_time'),
      u'ParseApplicationUsageRow')]

  # The required tables.
  REQUIRED_TABLES = frozenset([u'application_usage'])

  SCHEMAS = [{
      u'application_usage': (
          u'CREATE TABLE application_usage (event TEXT, bundle_id TEXT, '
          u'app_version TEXT, app_path TEXT, last_time INTEGER DEFAULT 0, '
          u'number_times INTEGER DEFAULT 0, PRIMARY KEY (event, bundle_id))')}]

  def ParseApplicationUsageRow(
      self, parser_mediator, row, query=None, **unused_kwargs):
    """Parses an application usage row.

    Args:
      parser_mediator (ParserMediator): mediates interactions between parsers
          and other components, such as storage and dfvfs.
      row (sqlite3.Row): row.
      query (Optional[str]): query.
    """
    # Note that pysqlite does not accept a Unicode string in row['string'] and
    # will raise "IndexError: Index must be int or string".

    # TODO: replace usage by definition(s) in eventdata. Not sure which values
    # it will hold here.
    usage = u'Application {0:s}'.format(row['event'])

    event_data = MacOSXApplicationUsageEventData()
    event_data.application = row['app_path']
    event_data.app_version = row['app_version']
    event_data.bundle_id = row['bundle_id']
    event_data.count = row['number_times']
    event_data.query = query

    date_time = dfdatetime_posix_time.PosixTime(timestamp=row['last_time'])
    event = time_events.DateTimeValuesEvent(date_time, usage)
    parser_mediator.ProduceEventWithEventData(event, event_data)


sqlite.SQLiteParser.RegisterPlugin(ApplicationUsagePlugin)



#!/usr/bin/python
# -*- coding: utf-8 -*-
"""Tests for the Chrome Preferences file parser."""

import unittest

from plaso.formatters import chrome_preferences  # pylint: disable=unused-import
from plaso.lib import timelib
from plaso.parsers import chrome_preferences

from tests import test_lib as shared_test_lib
from tests.parsers import test_lib


class ChromePreferencesParserTest(test_lib.ParserTestCase):
  """Tests for the Google Chrome Preferences file parser."""

  @shared_test_lib.skipUnlessHasTestFile([u'Preferences'])
  def testParseFile(self):
    """Tests parsing a default profile Preferences file."""
    parser = chrome_preferences.ChromePreferencesParser()
    storage_writer = self._ParseFile(
        [u'Preferences'], parser)

    self.assertEqual(storage_writer.number_of_events, 30)

    events = list(storage_writer.GetSortedEvents())

    event = events[17]
    expected_timestamp = timelib.Timestamp.CopyFromString(
        u'2014-11-12 13:01:43.926143')
    self.assertEqual(event.timestamp, expected_timestamp)
    expected_message = u'Chrome extensions autoupdater last run'
    expected_short_message = u'Chrome extensions autoupdater last run'
    self._TestGetMessageStrings(
        event, expected_message, expected_short_message)

    event = events[18]
    expected_timestamp = timelib.Timestamp.CopyFromString(
        u'2014-11-12 18:20:21.519200')
    self.assertEqual(event.timestamp, expected_timestamp)
    expected_message = u'Chrome extensions autoupdater next run'
    expected_short_message = u'Chrome extensions autoupdater next run'
    self._TestGetMessageStrings(
        event, expected_message, expected_short_message)

    event = events[22]
    expected_timestamp = timelib.Timestamp.CopyFromString(
        u'2016-06-08 16:17:47.453766')
    self.assertEqual(event.timestamp, expected_timestamp)
    expected_message = u'Chrome history was cleared by user'
    expected_short_message = u'Chrome history was cleared by user'
    self._TestGetMessageStrings(
        event, expected_message, expected_short_message)

    event = events[6]

    self.assertEqual(
        event.data_type, u'chrome:preferences:extension_installation')

    expected_timestamp = timelib.Timestamp.CopyFromString(
        u'2014-11-05 18:31:24.154837')
    self.assertEqual(event.timestamp, expected_timestamp)

    expected_id = u'mgndgikekgjfcpckkfioiadnlibdjbkf'
    self.assertEqual(event.extension_id, expected_id)

    expected_name = u'Chrome'
    self.assertEqual(event.extension_name, expected_name)

    expected_path = (
        u'C:\\Program Files\\Google\\Chrome\\Application\\38.0.2125.111\\'
        u'resources\\chrome_app')
    self.assertEqual(event.path, expected_path)

    expected_message = (
        u'CRX ID: {0:s} CRX Name: {1:s} Path: {2:s}'.format(
            expected_id, expected_name, expected_path))
    expected_short_message = (
        u'{0:s} '
        u'C:\\Program Files\\Google\\Chrome\\Application\\3...').format(
            expected_id)
    self._TestGetMessageStrings(
        event, expected_message, expected_short_message)

    event = events[25]
    expected_timestamp = timelib.Timestamp.CopyFromString(
        u'2016-11-14 14:12:50.588974')
    self.assertEqual(event.timestamp, expected_timestamp)
    expected_message = u'Permission geolocation used by local file'
    self._TestGetMessageStrings(
        event, expected_message, expected_message)

    event = events[23]
    expected_timestamp = timelib.Timestamp.CopyFromString(
        u'2016-11-11 16:20:09.866137')
    self.assertEqual(event.timestamp, expected_timestamp)
    expected_message = (
        u'Permission midi_sysex used by https://rawgit.com:443')
    expected_short_message = expected_message
    self._TestGetMessageStrings(
        event, expected_message, expected_short_message)

    event = events[29]
    expected_timestamp = timelib.Timestamp.CopyFromString(
        u'2016-11-14 14:13:00.639332')
    self.assertEqual(event.timestamp, expected_timestamp)
    expected_message = (
        u'Permission notifications used by https://rawgit.com:443')
    expected_short_message = (
        u'Permission notifications used by https://rawgit.com:443')
    self._TestGetMessageStrings(
        event, expected_message, expected_short_message)

    event = events[28]
    expected_timestamp = timelib.Timestamp.CopyFromString(
        u'2016-11-14 14:13:00.627093')
    self.assertEqual(event.timestamp, expected_timestamp)
    expected_message = (
        u'Permission notifications used by https://rawgit.com:443')
    expected_short_message = expected_message
    self._TestGetMessageStrings(
        event, expected_message, expected_short_message)

    event = events[27]
    expected_timestamp = timelib.Timestamp.CopyFromString(
        u'2016-11-14 14:12:54.899474')
    self.assertEqual(event.timestamp, expected_timestamp)
    expected_message = (
        u'Permission media_stream_mic used by local file')
    expected_short_message = expected_message
    self._TestGetMessageStrings(
        event, expected_message, expected_short_message)

    event = events[26]
    expected_timestamp = timelib.Timestamp.CopyFromString(
        u'2016-11-14 14:12:53.667838')
    self.assertEqual(event.timestamp, expected_timestamp)
    expected_message = (
        u'Permission media_stream_mic used by https://rawgit.com:443')
    expected_short_message = expected_message
    self._TestGetMessageStrings(
        event, expected_message, expected_short_message)


if __name__ == '__main__':
  unittest.main()



# -*- coding: utf-8 -*-
"""SQLite-based storage."""

import logging
import os
import sqlite3
import zlib

from plaso.containers import sessions
from plaso.lib import definitions
from plaso.storage import identifiers
from plaso.storage import interface


class SQLiteStorageFile(interface.BaseFileStorage):
  """SQLite-based storage file.

  Attributes:
    format_version (int): storage format version.
    serialization_format (str): serialization format.
    storage_type (str): storage type.
  """

  _FORMAT_VERSION = 20170707

  # The earliest format version, stored in-file, that this class
  # is able to read.
  _COMPATIBLE_FORMAT_VERSION = 20170707

  _CONTAINER_TYPES = (
      u'analysis_report', u'extraction_error', u'event', u'event_data',
      u'event_source', u'event_tag', u'session_completion', u'session_start',
      u'system_configuration', u'task_completion', u'task_start')

  _CREATE_METADATA_TABLE_QUERY = (
      u'CREATE TABLE metadata (key TEXT, value TEXT);')

  _CREATE_TABLE_QUERY = (
      u'CREATE TABLE {0:s} ('
      u'_identifier INTEGER PRIMARY KEY AUTOINCREMENT,'
      u'_data {1:s});')

  _CREATE_EVENT_TABLE_QUERY = (
      u'CREATE TABLE {0:s} ('
      u'_identifier INTEGER PRIMARY KEY AUTOINCREMENT,'
      u'_timestamp INTEGER,'
      u'_data {1:s});')

  _HAS_TABLE_QUERY = (
      u'SELECT name FROM sqlite_master '
      u'WHERE type = "table" AND name = "{0:s}"')

  # The maximum buffer size of serialized data before triggering
  # a flush to disk (64 MiB).
  _MAXIMUM_BUFFER_SIZE = 64 * 1024 * 1024

  def __init__(
      self, maximum_buffer_size=0,
      storage_type=definitions.STORAGE_TYPE_SESSION):
    """Initializes a storage.

    Args:
      maximum_buffer_size (Optional[int]):
          maximum size of a single storage stream. A value of 0 indicates
          the limit is _MAXIMUM_BUFFER_SIZE.
      storage_type (Optional[str]): storage type.

    Raises:
      ValueError: if the maximum buffer size value is out of bounds.
    """
    if (maximum_buffer_size < 0 or
        maximum_buffer_size > self._MAXIMUM_BUFFER_SIZE):
      raise ValueError(u'Maximum buffer size value out of bounds.')

    if not maximum_buffer_size:
      maximum_buffer_size = self._MAXIMUM_BUFFER_SIZE

    super(SQLiteStorageFile, self).__init__()
    self._connection = None
    self._cursor = None
    self._last_session = 0
    self._maximum_buffer_size = maximum_buffer_size

    if storage_type == definitions.STORAGE_TYPE_SESSION:
      self.compression_format = definitions.COMPRESSION_FORMAT_ZLIB
    else:
      self.compression_format = definitions.COMPRESSION_FORMAT_NONE

    self.format_version = self._FORMAT_VERSION
    self.serialization_format = definitions.SERIALIZER_FORMAT_JSON
    self.storage_type = storage_type

    # TODO: initialize next_sequence_number on read

  def _AddAttributeContainer(self, container_type, attribute_container):
    """Adds an attribute container.

    Args:
      container_type (str): attribute container type.
      attribute_container (AttributeContainer): attribute container.

    Raises:
      IOError: if the attribute container cannot be serialized.
    """
    container_list = self._GetSerializedAttributeContainerList(
        container_type)

    identifier = identifiers.SQLTableIdentifier(
        container_type, container_list.next_sequence_number + 1)
    attribute_container.SetIdentifier(identifier)

    serialized_data = self._SerializeAttributeContainer(attribute_container)

    container_list.PushAttributeContainer(serialized_data)

    if container_list.data_size > self._maximum_buffer_size:
      self._WriteSerializedAttributeContainerList(container_type)

  @classmethod
  def _CheckStorageMetadata(cls, metadata_values):
    """Checks the storage metadata.

    Args:
      metadata_values (dict[str, str]): metadata values per key.

    Raises:
      IOError: if the format version or the serializer format is not supported.
    """
    format_version = metadata_values.get(u'format_version', None)

    if not format_version:
      raise IOError(u'Missing format version.')

    try:
      format_version = int(format_version, 10)
    except (TypeError, ValueError):
      raise IOError(u'Invalid format version: {0!s}.'.format(format_version))

    if format_version < cls._COMPATIBLE_FORMAT_VERSION:
      raise IOError(
          u'Format version: {0:d} is too old and no longer supported.'.format(
              format_version))

    if format_version > cls._FORMAT_VERSION:
      raise IOError(
          u'Format version: {0:d} is too new and not yet supported.'.format(
              format_version))

    metadata_values[u'format_version'] = format_version

    compression_format = metadata_values.get(u'compression_format', None)
    if compression_format not in definitions.COMPRESSION_FORMATS:
      raise IOError(u'Unsupported compression format: {0:s}'.format(
          compression_format))

    serialization_format = metadata_values.get(u'serialization_format', None)
    if serialization_format != definitions.SERIALIZER_FORMAT_JSON:
      raise IOError(u'Unsupported serialization format: {0:s}'.format(
          serialization_format))

    storage_type = metadata_values.get(u'storage_type', None)
    if storage_type not in definitions.STORAGE_TYPES:
      raise IOError(u'Unsupported storage type: {0:s}'.format(
          storage_type))

  def _GetAttributeContainerByIndex(self, container_type, index):
    """Retrieves a specific attribute container.

    Args:
      container_type (str): attribute container type.
      index (int): attribute container index.

    Returnes:
      AttributeContainer: attribute container or None if not available.
    """
    sequence_number = index + 1
    query = u'SELECT _data FROM {0:s} WHERE rowid = {1:d}'.format(
        container_type, sequence_number)
    self._cursor.execute(query)

    row = self._cursor.fetchone()
    if row:
      identifier = identifiers.SQLTableIdentifier(
          container_type, sequence_number)

      if self.compression_format == definitions.COMPRESSION_FORMAT_ZLIB:
        serialized_data = zlib.decompress(row[0])
      else:
        serialized_data = row[0]

      attribute_container = self._DeserializeAttributeContainer(
          container_type, serialized_data)
      attribute_container.SetIdentifier(identifier)
      return attribute_container

    query = u'SELECT COUNT(*) FROM {0:s}'.format(container_type)
    self._cursor.execute(query)

    row = self._cursor.fetchone()
    index -= row[0]

    serialized_data = self._GetSerializedAttributeContainerByIndex(
        container_type, index)
    attribute_container = self._DeserializeAttributeContainer(
        container_type, serialized_data)

    if attribute_container:
      identifier = identifiers.SQLTableIdentifier(
          container_type, sequence_number)
      attribute_container.SetIdentifier(identifier)
    return attribute_container

  def _GetAttributeContainers(
      self, container_type, filter_expression=None, order_by=None):
    """Retrieves attribute containers.

    Args:
      container_type (str): attribute container type.
      filter_expression (Optional[str]): expression to filter results by.
      order_by (Optional[str]): name of a column to order the results by.

    Yields:
      AttributeContainer: attribute container.
    """
    query = u'SELECT _identifier, _data FROM {0:s}'.format(container_type)
    if filter_expression:
      query = u'{0:s} WHERE {1:s}'.format(query, filter_expression)
    if order_by:
      query = u'{0:s} ORDER BY {1:s}'.format(query, order_by)

    # Use a local cursor to prevent another query interrupting the generator.
    cursor = self._connection.cursor()

    cursor.execute(query)

    row = cursor.fetchone()
    while row:
      identifier = identifiers.SQLTableIdentifier(container_type, row[0])

      if self.compression_format == definitions.COMPRESSION_FORMAT_ZLIB:
        serialized_data = zlib.decompress(row[1])
      else:
        serialized_data = row[1]

      attribute_container = self._DeserializeAttributeContainer(
          container_type, serialized_data)
      attribute_container.SetIdentifier(identifier)
      yield attribute_container

      row = cursor.fetchone()

  def _HasTable(self, table_name):
    """Determines if a specific table exists.

    Args:
      table_name (str): name of the table.

    Returns:
      True if the table exists, false otheriwse.
    """
    query = self._HAS_TABLE_QUERY.format(table_name)

    self._cursor.execute(query)
    return bool(self._cursor.fetchone())

  def _ReadStorageMetadata(self):
    """Reads the storage metadata.

    Returns:
      bool: True if the storage metadata was read.
    """
    query = u'SELECT key, value FROM metadata'
    self._cursor.execute(query)

    metadata_values = {row[0]: row[1] for row in self._cursor.fetchall()}

    SQLiteStorageFile._CheckStorageMetadata(metadata_values)

    self.format_version = metadata_values[u'format_version']
    self.compression_format = metadata_values[u'compression_format']
    self.serialization_format = metadata_values[u'serialization_format']
    self.storage_type = metadata_values[u'storage_type']

  def _WriteAttributeContainer(self, attribute_container):
    """Writes an attribute container.

    The table for the container type must exist.

    Args:
      attribute_container (AttributeContainer): attribute container.
    """
    serialized_data = self._SerializeAttributeContainer(attribute_container)

    if self.compression_format == definitions.COMPRESSION_FORMAT_ZLIB:
      serialized_data = zlib.compress(serialized_data)
      serialized_data = sqlite3.Binary(serialized_data)

    query = u'INSERT INTO {0:s} (_data) VALUES (?)'.format(
        attribute_container.CONTAINER_TYPE)
    self._cursor.execute(query, (serialized_data, ))

    identifier = identifiers.SQLTableIdentifier(
        attribute_container.CONTAINER_TYPE, self._cursor.lastrowid)
    attribute_container.SetIdentifier(identifier)

  def _WriteSerializedAttributeContainerList(self, container_type):
    """Writes a serialized attribute container list.

    Args:
      container_type (str): attribute container type.
    """
    container_list = self._GetSerializedAttributeContainerList(container_type)
    if not container_list.data_size:
      return

    if self._serializers_profiler:
      self._serializers_profiler.StartTiming(u'write')

    query = u'INSERT INTO {0:s} (_data) VALUES (?)'.format(container_type)

    # TODO: directly use container_list instead of values_tuple_list.
    values_tuple_list = []
    for _ in range(container_list.number_of_attribute_containers):
      serialized_data = container_list.PopAttributeContainer()

      if self.compression_format == definitions.COMPRESSION_FORMAT_ZLIB:
        serialized_data = zlib.compress(serialized_data)
        serialized_data = sqlite3.Binary(serialized_data)

      values_tuple_list.append((serialized_data, ))

    self._cursor.executemany(query, values_tuple_list)

    if self._serializers_profiler:
      self._serializers_profiler.StopTiming(u'write')

    container_list.Empty()

  def _WriteStorageMetadata(self):
    """Writes the storage metadata."""
    self._cursor.execute(self._CREATE_METADATA_TABLE_QUERY)

    query = u'INSERT INTO metadata (key, value) VALUES (?, ?)'

    key = u'format_version'
    value = u'{0:d}'.format(self._FORMAT_VERSION)
    self._cursor.execute(query, (key, value))

    key = u'compression_format'
    value = self.compression_format
    self._cursor.execute(query, (key, value))

    key = u'serialization_format'
    value = self.serialization_format
    self._cursor.execute(query, (key, value))

    key = u'storage_type'
    value = self.storage_type
    self._cursor.execute(query, (key, value))

  def AddAnalysisReport(self, analysis_report):
    """Adds an analysis report.

    Args:
      analysis_report (AnalysisReport): analysis report.

    Raises:
      IOError: when the storage file is closed or read-only.
    """
    self._RaiseIfNotWritable()

    self._WriteAttributeContainer(analysis_report)

  def AddError(self, error):
    """Adds an error.

    Args:
      error (ExtractionError): error.

    Raises:
      IOError: when the storage file is closed or read-only.
    """
    self._RaiseIfNotWritable()

    self._AddAttributeContainer(u'extraction_error', error)

  def AddEvent(self, event):
    """Adds an event.

    Args:
      event (EventObject): event.

    Raises:
      IOError: when the storage file is closed or read-only or
          if the event data identifier type is not supported.
    """
    self._RaiseIfNotWritable()

    # TODO: change to no longer allow event_data_identifier is None
    # after refactoring every parser to generate event data.
    event_data_identifier = event.GetEventDataIdentifier()
    if event_data_identifier:
      if not isinstance(event_data_identifier, identifiers.SQLTableIdentifier):
        raise IOError(u'Unsupported event data identifier type: {0:s}'.format(
            type(event_data_identifier)))

      event.event_data_row_identifier = event_data_identifier.row_identifier

    self._AddAttributeContainer(u'event', event)

  def AddEventData(self, event_data):
    """Adds event data.

    Args:
      event_data (EventData): event data.

    Raises:
      IOError: when the storage file is closed or read-only.
    """
    self._RaiseIfNotWritable()

    self._AddAttributeContainer(u'event_data', event_data)

  def AddEventSource(self, event_source):
    """Adds an event source.

    Args:
      event_source (EventSource): event source.

    Raises:
      IOError: when the storage file is closed or read-only.
    """
    self._RaiseIfNotWritable()

    self._AddAttributeContainer(u'event_source', event_source)

  def AddEventTag(self, event_tag):
    """Adds an event tag.

    Args:
      event_tag (EventTag): event tag.

    Raises:
      IOError: when the storage file is closed or read-only or
          if the event identifier type is not supported.
    """
    self._RaiseIfNotWritable()

    event_identifier = event_tag.GetEventIdentifier()
    if not isinstance(event_identifier, identifiers.SQLTableIdentifier):
      raise IOError(u'Unsupported event identifier type: {0:s}'.format(
          type(event_identifier)))

    event_tag.event_row_identifier = event_identifier.row_identifier

    self._AddAttributeContainer(u'event_tag', event_tag)

  @classmethod
  def CheckSupportedFormat(cls, path):
    """Checks is the storage file format is supported.

    Args:
      path (str): path to the storage file.

    Returns:
      bool: True if the format is supported.
    """
    try:
      connection = sqlite3.connect(
          path, detect_types=sqlite3.PARSE_DECLTYPES|sqlite3.PARSE_COLNAMES)

      cursor = connection.cursor()

      query = u'SELECT * FROM metadata'
      cursor.execute(query)

      metadata_values = {row[0]: row[1] for row in cursor.fetchall()}

      cls._CheckStorageMetadata(metadata_values)

      connection.close()
      result = True

    except (IOError, sqlite3.DatabaseError):
      result = False

    return result

  def Close(self):
    """Closes the storage.

    Raises:
      IOError: if the storage file is already closed.
    """
    if not self._is_open:
      raise IOError(u'Storage file already closed.')

    if not self._read_only:
      self._WriteSerializedAttributeContainerList(u'event_source')
      self._WriteSerializedAttributeContainerList(u'event_data')
      self._WriteSerializedAttributeContainerList(u'event')
      self._WriteSerializedAttributeContainerList(u'event_tag')
      self._WriteSerializedAttributeContainerList(u'extraction_error')

    if self._serializers_profiler:
      self._serializers_profiler.Write()

    if self._connection:
      # We need to run commit or not all data is stored in the database.
      self._connection.commit()
      self._connection.close()

      self._connection = None
      self._cursor = None

    self._is_open = False

  def GetAnalysisReports(self):
    """Retrieves the analysis reports.

    Returns:
      generator(AnalysisReport): analysis report generator.
    """
    return self._GetAttributeContainers(u'analysis_report')

  def GetErrors(self):
    """Retrieves the errors.

    Returns:
      generator(ExtractionError): error generator.
    """
    return self._GetAttributeContainers(u'extraction_error')

  def GetEvents(self):
    """Retrieves the events.

    Yield:
      EventObject: event.
    """
    for event in self._GetAttributeContainers(u'event'):
      if hasattr(event, u'event_data_row_identifier'):
        event_data_identifier = identifiers.SQLTableIdentifier(
            u'event_data', event.event_data_row_identifier)
        event.SetEventDataIdentifier(event_data_identifier)

        del event.event_data_row_identifier

      yield event

  def GetEventData(self):
    """Retrieves the event data.

    Yields:
      generator(EventData): event data generator.
    """
    return self._GetAttributeContainers(u'event_data')

  def GetEventDataByIdentifier(self, identifier):
    """Retrieves specific event data.

    Args:
      identifier (SQLTableIdentifier): event data identifier.

    Returns:
      EventData: event data or None if not available.
    """
    return self._GetAttributeContainerByIndex(
        u'event_data', identifier.row_identifier - 1)

  def GetEventSourceByIndex(self, index):
    """Retrieves a specific event source.

    Args:
      index (int): event source index.

    Returns:
      EventSource: event source or None if not availabl.e
    """
    return self._GetAttributeContainerByIndex(u'event_source', index)

  def GetEventSources(self):
    """Retrieves the event sources.

    Yields:
      generator(EventSource): event source generator.
    """
    return self._GetAttributeContainers(u'event_source')

  def GetEventTags(self):
    """Retrieves the event tags.

    Yields:
      EventTag: event tag.
    """
    for event_tag in self._GetAttributeContainers(u'event_tag'):
      event_identifier = identifiers.SQLTableIdentifier(
          u'event', event_tag.event_row_identifier)
      event_tag.SetEventIdentifier(event_identifier)

      del event_tag.event_row_identifier

      yield event_tag

  def GetNumberOfAnalysisReports(self):
    """Retrieves the number analysis reports.

    Returns:
      int: number of analysis reports.
    """
    if not self._HasTable(u'analysis_reports'):
      return 0

    query = u'SELECT COUNT(*) FROM analysis_reports'
    self._cursor.execute(query)

    row = self._cursor.fetchone()
    return row[0]

  def GetNumberOfEventSources(self):
    """Retrieves the number event sources.

    Returns:
      int: number of event sources.
    """
    if not self._HasTable(u'event_source'):
      return 0

    query = u'SELECT COUNT(*) FROM event_source'
    self._cursor.execute(query)

    row = self._cursor.fetchone()
    number_of_event_sources = row[0]

    number_of_event_sources += self._GetNumberOfSerializedAttributeContainers(
        u'event_sources')
    return number_of_event_sources

  def GetSessions(self):
    """Retrieves the sessions.

    Yields:
      Session: session attribute container.

    Raises:
      IOError: if a stream is missing or there is a mismatch in session
          identifiers between the session start and completion attribute
          containers.
    """
    session_start_generator = self._GetAttributeContainers(u'session_start')
    session_completion_generator = self._GetAttributeContainers(
        u'session_completion')

    for session_index in range(0, self._last_session):
      session_start = next(session_start_generator)
      session_completion = next(session_completion_generator)

      session = sessions.Session()
      session.CopyAttributesFromSessionStart(session_start)
      if session_completion:
        try:
          session.CopyAttributesFromSessionCompletion(session_completion)
        except ValueError:
          raise IOError(
              u'Session identifier mismatch for session: {0:d}'.format(
                  session_index))

      yield session

  def GetSortedEvents(self, time_range=None):
    """Retrieves the events in increasing chronological order.

    Args:
      time_range (Optional[TimeRange]): time range used to filter events
          that fall in a specific period.

    Yield:
      EventObject: event.
    """
    filter_expression = None
    if time_range:
      filter_expression = []

      if time_range.start_timestamp:
        filter_expression.append(
            u'_timestamp >= {0:d}'.format(time_range.start_timestamp))

      if time_range.end_timestamp:
        filter_expression.append(
            u'_timestamp <= {0:d}'.format(time_range.end_timestamp))

      filter_expression = u' AND '.join(filter_expression)

    event_generator = self._GetAttributeContainers(
        u'event', filter_expression=filter_expression, order_by=u'_timestamp')

    for event in event_generator:
      if hasattr(event, u'event_data_row_identifier'):
        event_data_identifier = identifiers.SQLTableIdentifier(
            u'event_data', event.event_data_row_identifier)
        event.SetEventDataIdentifier(event_data_identifier)

        del event.event_data_row_identifier

      yield event

  def HasAnalysisReports(self):
    """Determines if a storage contains analysis reports.

    Returns:
      bool: True if the storage contains analysis reports.
    """
    query = u'SELECT COUNT(*) FROM analysis_report'
    self._cursor.execute(query)

    row = self._cursor.fetchone()
    return row and row[0] != 0

  def HasErrors(self):
    """Determines if a storage contains extraction errors.

    Returns:
      bool: True if the storage contains extraction errors.
    """
    query = u'SELECT COUNT(*) FROM extraction_error'
    self._cursor.execute(query)

    row = self._cursor.fetchone()
    return row and row[0] != 0

  def HasEventTags(self):
    """Determines if a storage contains event tags.

    Returns:
      bool: True if the storage contains event tags.
    """
    query = u'SELECT COUNT(*) FROM event_tags'
    self._cursor.execute(query)

    row = self._cursor.fetchone()
    return row and row[0] != 0

  # pylint: disable=arguments-differ
  def Open(self, path=None, read_only=True, **unused_kwargs):
    """Opens the storage.

    Args:
      path (Optional[str]): path to the storage file.
      read_only (Optional[bool]): True if the file should be opened in
          read-only mode.

    Raises:
      IOError: if the storage file is already opened or if the database
          cannot be connected.
      ValueError: if path is missing.
    """
    if self._is_open:
      raise IOError(u'Storage file already opened.')

    if not path:
      raise ValueError(u'Missing path.')

    connection = sqlite3.connect(
        path, detect_types=sqlite3.PARSE_DECLTYPES|sqlite3.PARSE_COLNAMES)

    cursor = connection.cursor()
    if not cursor:
      return False

    self._connection = connection
    self._cursor = cursor
    self._is_open = True
    self._read_only = read_only

    if not read_only:
      # self._cursor.execute(u'PRAGMA journal_mode=MEMORY')

      # Turn off insert transaction integrity since we want to do bulk insert.
      self._cursor.execute(u'PRAGMA synchronous=OFF')

      if not self._HasTable(u'metadata'):
        self._WriteStorageMetadata()
      else:
        self._ReadStorageMetadata()

      if self.compression_format == definitions.COMPRESSION_FORMAT_ZLIB:
        data_column_type = u'BLOB'
      else:
        data_column_type = u'TEXT'

      for container_type in self._CONTAINER_TYPES:
        if not self._HasTable(container_type):
          if container_type == u'event':
            query = self._CREATE_EVENT_TABLE_QUERY.format(
                container_type, data_column_type)
          else:
            query = self._CREATE_TABLE_QUERY.format(
                container_type, data_column_type)
          self._cursor.execute(query)

      self._connection.commit()

    last_session_start = 0
    if self._HasTable(u'session_start'):
      query = u'SELECT COUNT(*) FROM session_start'
      self._cursor.execute(query)
      row = self._cursor.fetchone()
      last_session_start = row[0]

    last_session_completion = 0
    if self._HasTable(u'session_completion'):
      query = u'SELECT COUNT(*) FROM session_completion'
      self._cursor.execute(query)
      row = self._cursor.fetchone()
      last_session_completion = row[0]

    # TODO: handle open sessions.
    if last_session_start != last_session_completion:
      logging.warning(u'Detected unclosed session.')

    self._last_session = last_session_completion

  def ReadPreprocessingInformation(self, knowledge_base):
    """Reads preprocessing information.

    The preprocessing information contains the system configuration which
    contains information about various system specific configuration data,
    for example the user accounts.

    Args:
      knowledge_base (KnowledgeBase): is used to store the preprocessing
          information.
    """
    generator = self._GetAttributeContainers(u'system_configuration')
    for stream_number, system_configuration in enumerate(generator):
      # TODO: replace stream_number by session_identifier.
      knowledge_base.ReadSystemConfigurationArtifact(
          system_configuration, session_identifier=stream_number)

  def WritePreprocessingInformation(self, knowledge_base):
    """Writes preprocessing information.

    Args:
      knowledge_base (KnowledgeBase): contains the preprocessing information.

    Raises:
      IOError: if the storage type does not support writing preprocess
          information or the storage file is closed or read-only.
    """
    self._RaiseIfNotWritable()

    if self.storage_type != definitions.STORAGE_TYPE_SESSION:
      raise IOError(u'Preprocess information not supported by storage type.')

    system_configuration = knowledge_base.GetSystemConfigurationArtifact()

    self._WriteAttributeContainer(system_configuration)

  def WriteSessionCompletion(self, session_completion):
    """Writes session completion information.

    Args:
      session_completion (SessionCompletion): session completion information.

    Raises:
      IOError: when the storage file is closed or read-only.
    """
    self._RaiseIfNotWritable()

    self._WriteAttributeContainer(session_completion)

  def WriteSessionStart(self, session_start):
    """Writes session start information.

    Args:
      session_start (SessionStart): session start information.

    Raises:
      IOError: when the storage file is closed or read-only.
    """
    self._RaiseIfNotWritable()

    self._WriteAttributeContainer(session_start)

  def WriteTaskCompletion(self, task_completion):
    """Writes task completion information.

    Args:
      task_completion (TaskCompletion): task completion information.

    Raises:
      IOError: when the storage file is closed or read-only.
    """
    self._RaiseIfNotWritable()

    self._WriteAttributeContainer(task_completion)

  def WriteTaskStart(self, task_start):
    """Writes task start information.

    Args:
      task_start (TaskStart): task start information.

    Raises:
      IOError: when the storage file is closed or read-only.
    """
    self._RaiseIfNotWritable()

    self._WriteAttributeContainer(task_start)


class SQLiteStorageMergeReader(interface.FileStorageMergeReader):
  """SQLite-based storage file reader for merging."""

  _CONTAINER_TYPES = (
      u'event_source', u'event_data', u'event', u'event_tag',
      u'extraction_error', u'analysis_report')

  _TABLE_NAMES_QUERY = (
      u'SELECT name FROM sqlite_master WHERE type = "table"')

  def __init__(self, storage_writer, path):
    """Initializes a storage merge reader.

    Args:
      storage_writer (StorageWriter): storage writer.
      path (str): path to the input file.

    Raises:
      IOError: if the input file cannot be opened.
    """
    super(SQLiteStorageMergeReader, self).__init__(storage_writer)
    self._active_container_type = None
    self._active_cursor = None
    self._connection = None
    self._container_types = None
    self._cursor = None
    self._event_data_identifier_mappings = {}
    self._path = path

  def _AddAttributeContainer(self, attribute_container):
    """Adds a single attribute container to the storage writer.

    Args:
      attribute_container (AttributeContainer): container

    Raises:
      RuntimeError: if the attribute container type is not supported.
    """
    container_type = attribute_container.CONTAINER_TYPE
    if container_type == u'event_source':
      self._storage_writer.AddEventSource(attribute_container)

    elif container_type == u'event_data':
      identifier = attribute_container.GetIdentifier()
      lookup_key = identifier.CopyToString()

      self._storage_writer.AddEventData(attribute_container)

      identifier = attribute_container.GetIdentifier()
      self._event_data_identifier_mappings[lookup_key] = identifier

    elif container_type == u'event':
      if hasattr(attribute_container, u'event_data_row_identifier'):
        event_data_identifier = identifiers.SQLTableIdentifier(
            u'event_data', attribute_container.event_data_row_identifier)
        lookup_key = event_data_identifier.CopyToString()

        event_data_identifier = self._event_data_identifier_mappings[lookup_key]
        attribute_container.SetEventDataIdentifier(event_data_identifier)

      # TODO: add event identifier mappings for event tags.

      self._storage_writer.AddEvent(attribute_container)

    elif container_type == u'event_tag':
      event_identifier = identifiers.SQLTableIdentifier(
          u'event', attribute_container.event_row_identifier)
      attribute_container.SetEventIdentifier(event_identifier)

      self._storage_writer.AddEventTag(attribute_container)

    elif container_type == u'extraction_error':
      self._storage_writer.AddError(attribute_container)

    elif container_type == u'analysis_report':
      self._storage_writer.AddAnalysisReport(attribute_container)

    elif container_type not in (u'task_completion', u'task_start'):
      raise RuntimeError(u'Unsupported container type: {0:s}'.format(
          container_type))

  def MergeAttributeContainers(self, maximum_number_of_containers=0):
    """Reads attribute containers from a task storage file into the writer.

    Args:
      maximum_number_of_containers (Optional[int]): maximum number of
          containers to merge, where 0 represent no limit.

    Returns:
      bool: True if the entire task storage file has been merged.

    Raises:
      OSError: if the task storage file cannot be deleted.
    """
    if not self._cursor:
      self._connection = sqlite3.connect(
          self._path,
          detect_types=sqlite3.PARSE_DECLTYPES|sqlite3.PARSE_COLNAMES)
      self._cursor = self._connection.cursor()

      self._cursor.execute(self._TABLE_NAMES_QUERY)
      table_names = [row[0] for row in self._cursor.fetchall()]

      # Remove container types not stored in the storage file but keep
      # the container types list in order.
      self._container_types = list(self._CONTAINER_TYPES)
      for name in set(self._CONTAINER_TYPES).difference(table_names):
        self._container_types.remove(name)

    number_of_containers = 0
    while self._active_cursor or self._container_types:
      if not self._active_cursor:
        self._active_container_type = self._container_types.pop(0)

        query = u'SELECT _identifier, _data FROM {0:s}'.format(
            self._active_container_type)
        self._cursor.execute(query)

        self._active_cursor = self._cursor

      if maximum_number_of_containers > 0:
        number_of_rows = maximum_number_of_containers - number_of_containers
        rows = self._active_cursor.fetchmany(size=number_of_rows)
      else:
        rows = self._active_cursor.fetchall()

      if not rows:
        self._active_cursor = None
        continue

      for row in rows:
        identifier = identifiers.SQLTableIdentifier(
            self._active_container_type, row[0])

        serialized_data = row[1]

        attribute_container = self._DeserializeAttributeContainer(
            self._active_container_type, serialized_data)
        attribute_container.SetIdentifier(identifier)

        self._AddAttributeContainer(attribute_container)

        number_of_containers += 1

      if (maximum_number_of_containers > 0 and
          number_of_containers >= maximum_number_of_containers):
        return False

    self._connection.close()
    self._connection = None
    self._cursor = None

    os.remove(self._path)

    return True


class SQLiteStorageFileReader(interface.FileStorageReader):
  """SQLite-based storage file reader."""

  def __init__(self, path):
    """Initializes a storage reader.

    Args:
      path (str): path to the input file.
    """
    super(SQLiteStorageFileReader, self).__init__(path)
    self._storage_file = SQLiteStorageFile()
    self._storage_file.Open(path=path)


class SQLiteStorageFileWriter(interface.FileStorageWriter):
  """SQLite-based storage file writer."""

  def _CreateStorageFile(self):
    """Creates a storage file.

    Returns:
      SQLiteStorageFile: storage file.
    """
    return SQLiteStorageFile(storage_type=self._storage_type)

  def _CreateTaskStorageMergeReader(self, path):
    """Creates a task storage merge reader.

    Args:
      path (str): path to the task storage file that should be merged.

    Returns:
      SQLiteStorageMergeReader: storage merge reader.
    """
    return SQLiteStorageMergeReader(self, path)

  def _CreateTaskStorageWriter(self, path, task):
    """Creates a task storage writer.

    Args:
      path (str): path to the storage file.
      task (Task): task.

    Returns:
      SQLiteStorageFileWriter: storage writer.
    """
    return SQLiteStorageFileWriter(
        self._session, path,
        storage_type=definitions.STORAGE_TYPE_TASK, task=task)



# -*- coding: utf-8 -*-
"""Plist plugin related functions and classes for testing."""

from plaso.storage import fake_storage

from plaso.containers import sessions
from plaso.parsers import plist
from tests.parsers import test_lib


class PlistPluginTestCase(test_lib.ParserTestCase):
  """The unit test case for a plist plugin."""

  def _ParsePlistFileWithPlugin(
      self, plugin, path_segments, plist_name,
      knowledge_base_values=None):
    """Parses a file using the parser and plugin object.

    Args:
      plugin (PlistPlugin): a plist plugin.
      path_segments (list[str]): the path segments inside the test data
          directory to the test file.
      plist_name (str): name of the plist to parse.
      knowledge_base_values (Optional[dict[str, object]]): knowledge base
          values.

    Returns:
      FakeStorageWriter: a storage writer.
    """
    file_entry = self._GetTestFileEntry(path_segments)
    file_object = file_entry.GetFileObject()

    parser = plist.PlistParser()
    top_level_object = parser.GetTopLevel(file_object)
    self.assertIsNotNone(top_level_object)

    return self._ParsePlistWithPlugin(
        plugin, plist_name, top_level_object,
        knowledge_base_values=knowledge_base_values)

  def _ParsePlistWithPlugin(
      self, plugin, plist_name, top_level_object,
      knowledge_base_values=None):
    """Parses a plist using the plugin object.

    Args:
      plugin (PlistPlugin): a plist plugin.
      plist_name (str): name of the plist to parse.
      top_level_object (dict[str, object]): plist top-level key.
      knowledge_base_values (Optional[dict[str, object]]): knowledge base
          values.

    Returns:
      FakeStorageWriter: a storage writer.
    """
    session = sessions.Session()
    storage_writer = fake_storage.FakeStorageWriter(session)
    storage_writer.Open()

    parser_mediator = self._CreateParserMediator(
        storage_writer, knowledge_base_values=knowledge_base_values)

    plugin.Process(
        parser_mediator, plist_name=plist_name, top_level=top_level_object)

    return storage_writer



# -*- coding: utf-8 -*-
"""The Google Chrome Preferences file event formatter."""

from plaso.formatters import interface
from plaso.formatters import manager
from plaso.lib import errors


class ChromePreferencesClearHistoryEventFormatter(
    interface.ConditionalEventFormatter):
  """Formatter for Chrome history clearing events."""

  DATA_TYPE = u'chrome:preferences:clear_history'

  FORMAT_STRING_PIECES = [u'{message}']

  FORMAT_STRING_SHORT_PIECES = [u'{message}']

  SOURCE_LONG = u'Chrome History Deletion'
  SOURCE_SHORT = u'LOG'


class ChromeExtensionsAutoupdaterEvent(interface.ConditionalEventFormatter):
  """Formatter for Chrome Extensions Autoupdater events."""

  DATA_TYPE = u'chrome:preferences:extensions_autoupdater'

  FORMAT_STRING_PIECES = [u'{message}']

  FORMAT_STRING_SHORT_PIECES = [u'{message}']

  SOURCE_LONG = u'Chrome Extensions Autoupdater'
  SOURCE_SHORT = u'LOG'


class ChromeExtensionInstallationEventFormatter(
    interface.ConditionalEventFormatter):
  """Formatter for a Chrome extension installation event."""

  DATA_TYPE = u'chrome:preferences:extension_installation'

  FORMAT_STRING_PIECES = [
      u'CRX ID: {extension_id}',
      u'CRX Name: {extension_name}',
      u'Path: {path}']

  FORMAT_STRING_SHORT_PIECES = [
      u'{extension_id}',
      u'{path}']

  SOURCE_LONG = u'Chrome Extension Installation'
  SOURCE_SHORT = u'LOG'


class ChromeContentSettingsExceptionsFormatter(
    interface.ConditionalEventFormatter):
  """Formatter for a Chrome content_settings exceptions event."""

  DATA_TYPE = u'chrome:preferences:content_settings:exceptions'

  FORMAT_STRING_PIECES = [
      u'Permission {permission}',
      u'used by {subject}']

  FORMAT_STRING_SHORT_PIECES = [
      u'Permission {permission}',
      u'used by {subject}']

  SOURCE_LONG = u'Chrome Permission Event'
  SOURCE_SHORT = u'LOG'

  def GetMessages(self, unused_formatter_mediator, event):
    """Determines the formatted message strings for an event object.

    Args:
      formatter_mediator (FormatterMediator): mediates the interactions between
          formatters and other components, such as storage and Windows EventLog
          resources.
      event (EventObject): event.

    Returns:
      tuple(str, str): formatted message string and short message string.

    Raises:
      WrongFormatter: if the event object cannot be formatted by the formatter.
    """
    if self.DATA_TYPE != event.data_type:
      raise errors.WrongFormatter(u'Unsupported data type: {0:s}.'.format(
          event.data_type))

    event_values = event.CopyToDict()

    primary_url = event_values[u'primary_url']
    secondary_url = event_values[u'secondary_url']

    # There is apparently a bug, either in GURL.cc or
    # content_settings_pattern.cc where URLs with file:// scheme are stored in
    # the URL as an empty string, which is later detected as being Invalid, and
    # Chrome produces the following example logs:
    # content_settings_pref.cc(469)] Invalid pattern strings: https://a.com:443,
    # content_settings_pref.cc(295)] Invalid pattern strings: ,
    # content_settings_pref.cc(295)] Invalid pattern strings: ,*
    # More research needed, could be related to https://crbug.com/132659

    if primary_url == u'':
      subject = u'local file'

    elif primary_url == secondary_url or secondary_url == u'*':
      subject = primary_url

    elif secondary_url == u'':
      subject = u'{0:s} embedded in local file'.format(primary_url)

    else:
      subject = u'{0:s} embedded in {1:s}'.format(primary_url, secondary_url)

    event_values[u'subject'] = subject

    return self._ConditionalFormatMessages(event_values)


manager.FormattersManager.RegisterFormatters([
    ChromeContentSettingsExceptionsFormatter,
    ChromeExtensionsAutoupdaterEvent,
    ChromeExtensionInstallationEventFormatter,
    ChromePreferencesClearHistoryEventFormatter])



# -*- coding: utf-8 -*-
"""Storage media related attribute container definitions."""

from plaso.containers import interface
from plaso.containers import manager


class MountPoint(interface.AttributeContainer):
  """Mount point attribute container.

  Attributes:
    mount_path (str): path where the path specification is mounted, such as
        "/mnt/image" or "C:\\".
    path_spec (dfvfs.PathSpec): path specification.
  """
  CONTAINER_TYPE = u'mount_point'

  def __init__(self, mount_path=None, path_specification=None):
    """Initializes a mount point.

    Args:
      mount_path (Optional[str]): path where the path specification is mounted,
          such as "/mnt/image" or "C:\\".
      path_specification (Optional[dfvfs.PathSpec]): path specification.
    """
    super(MountPoint, self).__init__()
    self.mount_path = mount_path
    self.path_specification = path_specification


manager.AttributeContainersManager.RegisterAttributeContainer(MountPoint)



# -*- coding: utf-8 -*-
"""The 4n6time output modules shared CLI arguments helper."""

from plaso.lib import errors
from plaso.cli.helpers import interface
from plaso.output import shared_4n6time


class Shared4n6TimeOutputArgumentsHelper(interface.ArgumentsHelper):
  """4n6time output modules shared CLI arguments helper."""

  NAME = u'4n6time'
  CATEGORY = u'output'
  DESCRIPTION = u'Argument helper for shared 4n6Time output modules.'

  _DEFAULT_APPEND = False
  _DEFAULT_EVIDENCE = u'-'
  _DEFAULT_FIELDS = u','.join([
      u'datetime', u'host', u'source', u'sourcetype', u'user', u'type'])

  @classmethod
  def AddArguments(cls, argument_group):
    """Adds command line arguments the helper supports to an argument group.

    This function takes an argument parser or an argument group object and adds
    to it all the command line arguments this helper supports.

    Args:
      argument_group (argparse._ArgumentGroup|argparse.ArgumentParser):
          argparse group.
    """
    argument_group.add_argument(
        u'--append', dest=u'append', action=u'store_true', default=False,
        required=cls._DEFAULT_APPEND, help=(
            u'Defines whether the intention is to append to an already '
            u'existing database or overwrite it. Defaults to overwrite.'))
    argument_group.add_argument(
        u'--evidence', dest=u'evidence', type=str,
        default=cls._DEFAULT_EVIDENCE, action=u'store', required=False,
        help=u'Set the evidence field to a specific value, defaults to empty.')
    argument_group.add_argument(
        u'--fields', dest=u'fields', type=str, action=u'store',
        default=cls._DEFAULT_FIELDS, help=(
            u'Defines which fields should be indexed in the database.'))
    argument_group.add_argument(
        u'--additional_fields', dest=u'additional_fields', type=str,
        action=u'store', default=u'', help=(
            u'Defines extra fields to be included in the output, in addition to'
            u' the default fields, which are {0:s}.'.format(
                cls._DEFAULT_FIELDS)))

  @classmethod
  def ParseOptions(cls, options, output_module):
    """Parses and validates options.

    Args:
      options (argparse.Namespace): parser options.
      output_module (OutputModule): output module to configure.

    Raises:
      BadConfigObject: when the output module object is of the wrong type.
    """
    if not isinstance(output_module, shared_4n6time.Shared4n6TimeOutputModule):
      raise errors.BadConfigObject(
          u'Output module is not an instance of Shared4n6TimeOutputModule')

    append = getattr(options, u'append', cls._DEFAULT_APPEND)
    evidence = cls._ParseStringOption(
        options, u'evidence', default_value=cls._DEFAULT_EVIDENCE)
    fields = cls._ParseStringOption(
        options, u'fields', default_value=cls._DEFAULT_FIELDS)
    additional_fields = cls._ParseStringOption(
        options, u'additional_fields')

    if additional_fields:
      fields = u'{0:s},{1:s}'.format(fields, additional_fields)

    output_module.SetAppendMode(append)
    output_module.SetEvidence(evidence)
    output_module.SetFields([
        field_name.strip() for field_name in fields.split(u',')])



# -*- coding: utf-8 -*-
"""Browser cookie plugin related functions and classes for testing."""

from tests.parsers import test_lib


class CookiePluginTestCase(test_lib.ParserTestCase):
  """The unit test case for a browser cookie plugin."""



# -*- coding: utf-8 -*-
"""The storage media CLI tool."""

import getpass
import logging
import os
import sys

from dfdatetime import filetime as dfdatetime_filetime
from dfvfs.analyzer import analyzer as dfvfs_analyzer
from dfvfs.analyzer import fvde_analyzer_helper
from dfvfs.credentials import manager as credentials_manager
from dfvfs.helpers import source_scanner
from dfvfs.lib import definitions as dfvfs_definitions
from dfvfs.lib import errors as dfvfs_errors
from dfvfs.path import factory as path_spec_factory
from dfvfs.volume import tsk_volume_system
from dfvfs.volume import vshadow_volume_system

from plaso.cli import tools
from plaso.engine import configurations
from plaso.lib import errors
from plaso.lib import py2to3
from plaso.lib import timelib


try:
  # Disable experimental FVDE support.
  dfvfs_analyzer.Analyzer.DeregisterHelper(
      fvde_analyzer_helper.FVDEAnalyzerHelper())
except KeyError:
  pass


class StorageMediaTool(tools.CLITool):
  """Class that implements a storage media CLI tool."""

  _DEFAULT_BYTES_PER_SECTOR = 512

  # TODO: remove this redirect.
  _SOURCE_OPTION = u'source'

  _BINARY_DATA_CREDENTIAL_TYPES = [u'key_data']

  _SUPPORTED_CREDENTIAL_TYPES = [
      u'key_data', u'password', u'recovery_password', u'startup_key']

  # For context see: http://en.wikipedia.org/wiki/Byte
  _UNITS_1000 = [u'B', u'kB', u'MB', u'GB', u'TB', u'EB', u'ZB', u'YB']
  _UNITS_1024 = [u'B', u'KiB', u'MiB', u'GiB', u'TiB', u'EiB', u'ZiB', u'YiB']

  def __init__(self, input_reader=None, output_writer=None):
    """Initializes the CLI tool object.

    Args:
      input_reader (Optional[InputReader]): input reader, where None indicates
          that the stdin input reader should be used.
      output_writer (Optional[OutputWriter]): output writer, where None
          indicates that the stdout output writer should be used.
    """
    super(StorageMediaTool, self).__init__(
        input_reader=input_reader, output_writer=output_writer)
    self._credentials = []
    self._credential_configurations = []
    self._filter_file = None
    self._partitions = None
    self._partition_offset = None
    self._process_vss = False
    self._source_scanner = source_scanner.SourceScanner()
    self._source_path = None
    self._source_path_specs = []
    self._vss_only = False
    self._vss_stores = None

  def _AddCredentialConfiguration(
      self, path_spec, credential_type, credential_data):
    """Adds a credential configuration.

    Args:
      path_spec (dfvfs.PathSpec): path specification.
      credential_type (str): credential type.
      credential_data (bytes): credential data.
    """
    credential_configuration = configurations.CredentialConfiguration(
        credential_data=credential_data, credential_type=credential_type,
        path_spec=path_spec)

    self._credential_configurations.append(credential_configuration)

  def _FormatHumanReadableSize(self, size):
    """Represents a number of bytes as a human readable string.

    Args:
      size (int): size in bytes.

    Returns:
      str: human readable string of the size.
    """
    magnitude_1000 = 0
    size_1000 = float(size)
    while size_1000 >= 1000:
      size_1000 /= 1000
      magnitude_1000 += 1

    magnitude_1024 = 0
    size_1024 = float(size)
    while size_1024 >= 1024:
      size_1024 /= 1024
      magnitude_1024 += 1

    size_string_1000 = None
    if magnitude_1000 > 0 and magnitude_1000 <= 7:
      size_string_1000 = u'{0:.1f}{1:s}'.format(
          size_1000, self._UNITS_1000[magnitude_1000])

    size_string_1024 = None
    if magnitude_1024 > 0 and magnitude_1024 <= 7:
      size_string_1024 = u'{0:.1f}{1:s}'.format(
          size_1024, self._UNITS_1024[magnitude_1024])

    if not size_string_1000 or not size_string_1024:
      return u'{0:d} B'.format(size)

    return u'{0:s} / {1:s} ({2:d} B)'.format(
        size_string_1024, size_string_1000, size)

  def _GetNormalizedTSKVolumeIdentifiers(
      self, volume_system, volume_identifiers):
    """Retrieves the normalized TSK volume identifiers.

    Args:
      volume_system (dfvfs.TSKVolumeSystem): volume system.
      volume_identifiers (list[str]): allowed volume identifiers.

    Returns:
      list[int]: normalized volume identifiers.
    """
    normalized_volume_identifiers = []
    for volume_identifier in volume_identifiers:
      volume = volume_system.GetVolumeByIdentifier(volume_identifier)
      if not volume:
        raise errors.SourceScannerError(
            u'Volume missing for identifier: {0:s}.'.format(volume_identifier))

      try:
        volume_identifier = int(volume.identifier[1:], 10)
        normalized_volume_identifiers.append(volume_identifier)
      except ValueError:
        pass

    return normalized_volume_identifiers

  def _GetNormalizedVShadowVolumeIdentifiers(
      self, volume_system, volume_identifiers):
    """Retrieves the normalized VShadow volume identifiers.

    Args:
      volume_system (dfvfs.VShadowVolumeSystem): volume system.
      volume_identifiers (list[str]): allowed volume identifiers.

    Returns:
      list[int]: normalized volume identifiers.
    """
    normalized_volume_identifiers = []
    for volume_identifier in volume_identifiers:
      volume = volume_system.GetVolumeByIdentifier(volume_identifier)
      if not volume:
        raise errors.SourceScannerError(
            u'Volume missing for identifier: {0:s}.'.format(volume_identifier))

      try:
        volume_identifier = int(volume.identifier[3:], 10)
        normalized_volume_identifiers.append(volume_identifier)
      except ValueError:
        pass

    return normalized_volume_identifiers

  # TODO: refactor this method that it become more clear what it is
  # supposed to do.
  def _GetTSKPartitionIdentifiers(
      self, scan_node, partition_offset=None, partitions=None):
    """Determines the TSK partition identifiers.

    This method first checks for the preferred partition number, then for
    the preferred partition offset and falls back to prompt the user if
    no usable preferences were specified.

    Args:
      scan_node (dfvfs.SourceScanNode): scan node.
      partition_offset (Optional[int]): preferred partition byte offset.
      paritions (Optional[list[str]]): preferred partition identifiers.

    Returns:
      list[str]: partition identifiers.

    Raises:
      RuntimeError: if the volume for a specific identifier cannot be
          retrieved.
      SourceScannerError: if the format of or within the source
          is not supported or the the scan node is invalid.
    """
    if not scan_node or not scan_node.path_spec:
      raise errors.SourceScannerError(u'Invalid scan node.')

    volume_system = tsk_volume_system.TSKVolumeSystem()
    volume_system.Open(scan_node.path_spec)

    volume_identifiers = self._source_scanner.GetVolumeIdentifiers(
        volume_system)
    if not volume_identifiers:
      self._output_writer.Write(u'[WARNING] No partitions found.\n')
      return

    normalized_volume_identifiers = self._GetNormalizedTSKVolumeIdentifiers(
        volume_system, volume_identifiers)

    if partitions:
      if partitions == [u'all']:
        partitions = range(1, volume_system.number_of_volumes + 1)

      if not set(partitions).difference(normalized_volume_identifiers):
        return [
            u'p{0:d}'.format(partition_number)
            for partition_number in partitions]

    if partition_offset is not None:
      for volume in volume_system.volumes:
        volume_extent = volume.extents[0]
        if volume_extent.offset == partition_offset:
          return [volume.identifier]

      self._output_writer.Write((
          u'[WARNING] No such partition with offset: {0:d} '
          u'(0x{0:08x}).\n').format(partition_offset))

    if len(volume_identifiers) == 1:
      return volume_identifiers

    try:
      selected_volume_identifier = self._PromptUserForPartitionIdentifier(
          volume_system, volume_identifiers)
    except KeyboardInterrupt:
      raise errors.UserAbort(u'File system scan aborted.')

    if selected_volume_identifier == u'all':
      return volume_identifiers

    return [selected_volume_identifier]

  def _GetVSSStoreIdentifiers(self, scan_node, vss_stores=None):
    """Determines the VSS store identifiers.

    Args:
      scan_node (dfvfs.SourceScanNode): scan node.
      vss_stores (Optional[list[str]]): preferred VSS store identifiers.

    Returns:
      list[str] VSS store identifiers.

    Raises:
      SourceScannerError: if the format of or within the source
          is not supported or the the scan node is invalid.
    """
    if not scan_node or not scan_node.path_spec:
      raise errors.SourceScannerError(u'Invalid scan node.')

    volume_system = vshadow_volume_system.VShadowVolumeSystem()
    volume_system.Open(scan_node.path_spec)

    volume_identifiers = self._source_scanner.GetVolumeIdentifiers(
        volume_system)
    if not volume_identifiers:
      return []

    try:
      selected_store_identifiers = self._PromptUserForVSSStoreIdentifiers(
          volume_system, volume_identifiers, vss_stores=vss_stores)
    except KeyboardInterrupt:
      raise errors.UserAbort(u'File system scan aborted.')

    return selected_store_identifiers

  def _ParseCredentialOptions(self, options):
    """Parses the credential options.

    Args:
      options (argparse.Namespace): command line arguments.

    Raises:
      BadConfigOption: if the options are invalid.
    """
    credentials = getattr(options, u'credentials', [])
    if not isinstance(credentials, list):
      raise errors.BadConfigOption(u'Unsupported credentials value.')

    for credential_string in credentials:
      credential_type, _, credential_data = credential_string.partition(u':')
      if not credential_type or not credential_data:
        raise errors.BadConfigOption(
            u'Badly formatted credential: {0:s}.'.format(credential_string))

      if credential_type not in self._SUPPORTED_CREDENTIAL_TYPES:
        raise errors.BadConfigOption(
            u'Unsupported credential type for: {0:s}.'.format(
                credential_string))

      if credential_type in self._BINARY_DATA_CREDENTIAL_TYPES:
        try:
          credential_data = credential_data.decode(u'hex')
        except TypeError:
          raise errors.BadConfigOption(
              u'Unsupported credential data for: {0:s}.'.format(
                  credential_string))

      self._credentials.append((credential_type, credential_data))

  def _ParsePartitionsString(self, partitions):
    """Parses the user specified partitions string.

    Args:
      partitions (str): partitions. A range of partitions can be defined
          as: "3..5". Multiple partitions can be defined as: "1,3,5" (a list
          of comma separated values). Ranges and lists can also be combined
          as: "1,3..5". The first partition is 1. All partition can be
          defined as: "all".

    Returns:
      list[str]: partitions.

    Raises:
      BadConfigOption: if the partitions option is invalid.
    """
    if not partitions:
      return []

    if partitions == u'all':
      return [u'all']

    partition_numbers = []
    for partition_range in partitions.split(u','):
      # Determine if the range is formatted as 1..3 otherwise it indicates
      # a single partition number.
      if u'..' in partition_range:
        first_partition, last_partition = partition_range.split(u'..')
        try:
          first_partition = int(first_partition, 10)
          last_partition = int(last_partition, 10)
        except ValueError:
          raise errors.BadConfigOption(
              u'Invalid partition range: {0:s}.'.format(partition_range))

        for partition_number in range(first_partition, last_partition + 1):
          if partition_number not in partition_numbers:
            partition_numbers.append(partition_number)
      else:
        if partition_range.startswith(u'p'):
          partition_range = partition_range[1:]

        try:
          partition_number = int(partition_range, 10)
        except ValueError:
          raise errors.BadConfigOption(
              u'Invalid partition range: {0:s}.'.format(partition_range))

        if partition_number not in partition_numbers:
          partition_numbers.append(partition_number)

    return sorted(partition_numbers)

  def _ParseSourcePathOption(self, options):
    """Parses the source path option.

    Args:
      options (argparse.Namespace): command line arguments.

    Raises:
      BadConfigOption: if the options are invalid.
    """
    self._source_path = self.ParseStringOption(options, self._SOURCE_OPTION)
    if not self._source_path:
      raise errors.BadConfigOption(u'Missing source path.')

    self._source_path = os.path.abspath(self._source_path)

  def _ParseStorageMediaOptions(self, options):
    """Parses the storage media options.

    Args:
      options (argparse.Namespace): command line arguments.

    Raises:
      BadConfigOption: if the options are invalid.
    """
    self._ParseStorageMediaImageOptions(options)
    self._ParseVSSProcessingOptions(options)
    self._ParseCredentialOptions(options)
    self._ParseSourcePathOption(options)

  def _ParseStorageMediaImageOptions(self, options):
    """Parses the storage media image options.

    Args:
      options (argparse.Namespace): command line arguments.

    Raises:
      BadConfigOption: if the options are invalid.
    """
    partitions = getattr(options, u'partitions', None)
    self._partitions = self._ParsePartitionsString(partitions)

    partition = getattr(options, u'partition', None)

    if self._partitions and partition is not None:
      raise errors.BadConfigOption((
          u'Option "--partition" can not be used in combination '
          u'with "--partitions".'))

    if not self._partitions and partition is not None:
      self._partitions = self._ParsePartitionsString(partition)

    image_offset_bytes = getattr(options, u'image_offset_bytes', None)

    if self._partitions and image_offset_bytes is not None:
      raise errors.BadConfigOption((
          u'Option "--image_offset_bytes" can not be used in combination '
          u'with "--partitions" or "--partition".'))

    image_offset = getattr(options, u'image_offset', None)

    if self._partitions and image_offset is not None:
      raise errors.BadConfigOption((
          u'Option "--image_offset" can not be used in combination with '
          u'"--partitions" or "--partition".'))

    if (image_offset_bytes is not None and
        isinstance(image_offset_bytes, py2to3.STRING_TYPES)):
      try:
        image_offset_bytes = int(image_offset_bytes, 10)
      except ValueError:
        raise errors.BadConfigOption(
            u'Invalid image offset bytes: {0:s}.'.format(image_offset_bytes))

    if image_offset_bytes is None and image_offset is not None:
      bytes_per_sector = getattr(
          options, u'bytes_per_sector', self._DEFAULT_BYTES_PER_SECTOR)

      if isinstance(image_offset, py2to3.STRING_TYPES):
        try:
          image_offset = int(image_offset, 10)
        except ValueError:
          raise errors.BadConfigOption(
              u'Invalid image offset: {0:s}.'.format(image_offset))

      if isinstance(bytes_per_sector, py2to3.STRING_TYPES):
        try:
          bytes_per_sector = int(bytes_per_sector, 10)
        except ValueError:
          raise errors.BadConfigOption(
              u'Invalid bytes per sector: {0:s}.'.format(bytes_per_sector))

    if image_offset_bytes:
      self._partition_offset = image_offset_bytes
    elif image_offset:
      self._partition_offset = image_offset * bytes_per_sector

  def _ParseVSSProcessingOptions(self, options):
    """Parses the VSS processing options.

    Args:
      options (argparse.Namespace): command line arguments.

    Raises:
      BadConfigOption: if the options are invalid.
    """
    vss_only = False
    vss_stores = None

    self._process_vss = not getattr(options, u'no_vss', True)
    if self._process_vss:
      vss_only = getattr(options, u'vss_only', False)
      vss_stores = getattr(options, u'vss_stores', None)

    if vss_stores:
      vss_stores = self._ParseVSSStoresString(vss_stores)

    self._vss_only = vss_only
    self._vss_stores = vss_stores

  def _ParseVSSStoresString(self, vss_stores):
    """Parses the user specified VSS stores string.

    Args:
      vss_stores (str): VSS stores. A range of stores can be defined
          as: "3..5". Multiple stores can be defined as: "1,3,5" (a list
          of comma separated values). Ranges and lists can also be
          combined as: "1,3..5". The first store is 1. All stores can be
          defined as: "all".

    Returns:
      list[str]: VSS stores.

    Raises:
      BadConfigOption: if the VSS stores option is invalid.
    """
    if not vss_stores:
      return []

    if vss_stores == u'all':
      return [u'all']

    store_numbers = []
    for vss_store_range in vss_stores.split(u','):
      # Determine if the range is formatted as 1..3 otherwise it indicates
      # a single store number.
      if u'..' in vss_store_range:
        first_store, last_store = vss_store_range.split(u'..')
        try:
          first_store = int(first_store, 10)
          last_store = int(last_store, 10)
        except ValueError:
          raise errors.BadConfigOption(
              u'Invalid VSS store range: {0:s}.'.format(vss_store_range))

        for store_number in range(first_store, last_store + 1):
          if store_number not in store_numbers:
            store_numbers.append(store_number)
      else:
        if vss_store_range.startswith(u'vss'):
          vss_store_range = vss_store_range[3:]

        try:
          store_number = int(vss_store_range, 10)
        except ValueError:
          raise errors.BadConfigOption(
              u'Invalid VSS store range: {0:s}.'.format(vss_store_range))

        if store_number not in store_numbers:
          store_numbers.append(store_number)

    return sorted(store_numbers)

  def _PromptUserForEncryptedVolumeCredential(
      self, scan_context, locked_scan_node, credentials):
    """Prompts the user to provide a credential for an encrypted volume.

    Args:
      scan_context (dfvfs.SourceScannerContext): source scanner context.
      locked_scan_node (dfvfs.SourceScanNode): locked scan node.
      credentials (dfvfs.Credentials): credentials supported by the locked
          scan node.

    Returns:
      bool: True if the volume was unlocked.
    """
    # TODO: print volume description.
    if locked_scan_node.type_indicator == dfvfs_definitions.TYPE_INDICATOR_BDE:
      self._output_writer.Write(u'Found a BitLocker encrypted volume.\n')
    else:
      self._output_writer.Write(u'Found an encrypted volume.\n')

    credentials_list = list(credentials.CREDENTIALS)
    credentials_list.append(u'skip')

    self._output_writer.Write(u'Supported credentials:\n')
    self._output_writer.Write(u'\n')
    for index, name in enumerate(credentials_list):
      self._output_writer.Write(u'  {0:d}. {1:s}\n'.format(index, name))
    self._output_writer.Write(u'\nNote that you can abort with Ctrl^C.\n\n')

    result = False
    while not result:
      self._output_writer.Write(u'Select a credential to unlock the volume: ')
      # TODO: add an input reader.
      input_line = self._input_reader.Read()
      input_line = input_line.strip()

      if input_line in credentials_list:
        credential_type = input_line
      else:
        try:
          credential_type = int(input_line, 10)
          credential_type = credentials_list[credential_type]
        except (IndexError, ValueError):
          self._output_writer.Write(
              u'Unsupported credential: {0:s}\n'.format(input_line))
          continue

      if credential_type == u'skip':
        break

      getpass_string = u'Enter credential data: '
      if sys.platform.startswith(u'win') and sys.version_info[0] < 3:
        # For Python 2 on Windows getpass (win_getpass) requires an encoded
        # byte string. For Python 3 we need it to be a Unicode string.
        getpass_string = self._EncodeString(getpass_string)

      credential_data = getpass.getpass(getpass_string)
      self._output_writer.Write(u'\n')

      if credential_type in self._BINARY_DATA_CREDENTIAL_TYPES:
        try:
          credential_data = credential_data.decode(u'hex')
        except TypeError:
          self._output_writer.Write(u'Unsupported credential data.\n')
          continue

      try:
        result = self._source_scanner.Unlock(
            scan_context, locked_scan_node.path_spec, credential_type,
            credential_data)

      except IOError as exception:
        logging.debug(u'Unable to unlock volume with error: {0:s}'.format(
            exception))
        result = False

      if not result:
        self._output_writer.Write(u'Unable to unlock volume.\n')
        self._output_writer.Write(u'\n')

    self._output_writer.Write(u'\n')

    if result:
      self._AddCredentialConfiguration(
          locked_scan_node.path_spec, credential_type, credential_data)

    return result

  def _PromptUserForPartitionIdentifier(
      self, volume_system, volume_identifiers):
    """Prompts the user to provide a partition identifier.

    Args:
      volume_system (dfvfs.TSKVolumeSystem): volume system.
      volume_identifiers (list[str]): allowed volume identifiers.

    Returns:
      str: partition identifier or 'all'.

    Raises:
      SourceScannerError: if the source cannot be processed.
    """
    self._output_writer.Write(
        u'The following partitions were found:\n'
        u'Identifier\tOffset (in bytes)\tSize (in bytes)\n')

    for volume_identifier in sorted(volume_identifiers):
      volume = volume_system.GetVolumeByIdentifier(volume_identifier)
      if not volume:
        raise errors.SourceScannerError(
            u'Volume missing for identifier: {0:s}.'.format(volume_identifier))

      volume_extent = volume.extents[0]
      self._output_writer.Write(
          u'{0:s}\t\t{1:d} (0x{1:08x})\t{2:s}\n'.format(
              volume.identifier, volume_extent.offset,
              self._FormatHumanReadableSize(volume_extent.size)))

    self._output_writer.Write(u'\n')

    while True:
      self._output_writer.Write(
          u'Please specify the identifier of the partition that should be '
          u'processed.\nAll partitions can be defined as: "all". Note that you '
          u'can abort with Ctrl^C.\n')

      selected_volume_identifier = self._input_reader.Read()
      selected_volume_identifier = selected_volume_identifier.strip()

      if not selected_volume_identifier.startswith(u'p'):
        try:
          partition_number = int(selected_volume_identifier, 10)
          selected_volume_identifier = u'p{0:d}'.format(partition_number)
        except ValueError:
          pass

      if (selected_volume_identifier == u'all' or
          selected_volume_identifier in volume_identifiers):
        break

      self._output_writer.Write(
          u'\n'
          u'Unsupported partition identifier, please try again or abort '
          u'with Ctrl^C.\n'
          u'\n')

    self._output_writer.Write(u'\n')
    return selected_volume_identifier

  def _PromptUserForVSSCurrentVolume(self):
    """Prompts the user if the current volume with VSS should be processed.

    Returns:
      bool: True if the current volume with VSS should be processed.
    """
    while True:
      self._output_writer.Write(
          u'Volume Shadow Snapshots (VSS) were selected also process current\n'
          u'volume? [yes, no]\n')

      process_current_volume = self._input_reader.Read()
      process_current_volume = process_current_volume.strip()
      process_current_volume = process_current_volume.lower()

      if (not process_current_volume or
          process_current_volume in (u'no', u'yes')):
        break

      self._output_writer.Write(
          u'\n'
          u'Unsupported option, please try again or abort with Ctrl^C.\n'
          u'\n')

    self._output_writer.Write(u'\n')
    return not process_current_volume or process_current_volume == u'yes'

  def _PromptUserForVSSStoreIdentifiers(
      self, volume_system, volume_identifiers, vss_stores=None):
    """Prompts the user to provide the VSS store identifiers.

    This method first checks for the preferred VSS stores and falls back
    to prompt the user if no usable preferences were specified.

    Args:
      volume_system (dfvfs.VShadowVolumeSystem): volume system.
      volume_identifiers (list[str]): allowed volume identifiers.
      vss_stores (Optional[list[str]]): preferred VSS store identifiers.

    Returns:
      list[str]: selected VSS store identifiers.

    Raises:
      SourceScannerError: if the source cannot be processed.
    """
    normalized_volume_identifiers = self._GetNormalizedVShadowVolumeIdentifiers(
        volume_system, volume_identifiers)

    # TODO: refactor this to _GetVSSStoreIdentifiers.
    if vss_stores:
      if vss_stores == [u'all']:
        # We need to set the stores to cover all vss stores.
        vss_stores = range(1, volume_system.number_of_volumes + 1)

      if not set(vss_stores).difference(normalized_volume_identifiers):
        return vss_stores

    print_header = True
    while True:
      if print_header:
        self._output_writer.Write(
            u'The following Volume Shadow Snapshots (VSS) were found:\n'
            u'Identifier\t\tCreation Time\n')

        for volume_identifier in volume_identifiers:
          volume = volume_system.GetVolumeByIdentifier(volume_identifier)
          if not volume:
            raise errors.SourceScannerError(
                u'Volume missing for identifier: {0:s}.'.format(
                    volume_identifier))

          vss_creation_time = volume.GetAttribute(u'creation_time')
          filetime = dfdatetime_filetime.Filetime(
              timestamp=vss_creation_time.value)
          vss_creation_time = filetime.GetPlasoTimestamp()
          vss_creation_time = timelib.Timestamp.CopyToIsoFormat(
              vss_creation_time)

          if volume.HasExternalData():
            external_data = u'\tWARNING: data stored outside volume'
          else:
            external_data = u''

          self._output_writer.Write(u'{0:s}\t\t\t{1:s}{2:s}\n'.format(
              volume.identifier, vss_creation_time, external_data))

        self._output_writer.Write(u'\n')

        print_header = False

      self._output_writer.Write(
          u'Please specify the identifier(s) of the VSS that should be '
          u'processed:\nNote that a range of stores can be defined as: 3..5. '
          u'Multiple stores can\nbe defined as: 1,3,5 (a list of comma '
          u'separated values). Ranges and lists can\nalso be combined '
          u'as: 1,3..5. The first store is 1. All stores can be defined\n'
          u'as "all". If no stores are specified none will be processed. You\n'
          u'can abort with Ctrl^C.\n')

      selected_vss_stores = self._input_reader.Read()

      selected_vss_stores = selected_vss_stores.strip()
      if not selected_vss_stores:
        return []

      try:
        selected_vss_stores = self._ParseVSSStoresString(selected_vss_stores)
      except errors.BadConfigOption:
        selected_vss_stores = []

      if selected_vss_stores == [u'all']:
        # We need to set the stores to cover all vss stores.
        selected_vss_stores = range(1, volume_system.number_of_volumes + 1)

      if not set(selected_vss_stores).difference(normalized_volume_identifiers):
        break

      self._output_writer.Write(
          u'\n'
          u'Unsupported VSS identifier(s), please try again or abort with '
          u'Ctrl^C.\n'
          u'\n')

    self._output_writer.Write(u'\n')
    return selected_vss_stores

  def _ScanVolume(self, scan_context, volume_scan_node):
    """Scans the volume scan node for volume and file systems.

    Args:
      scan_context (dfvfs.SourceScannerContext): source scanner context.
      volume_scan_node (dfvfs.SourceScanNode): volume scan node.

    Raises:
      SourceScannerError: if the format of or within the source
          is not supported or the the scan node is invalid.
    """
    if not volume_scan_node or not volume_scan_node.path_spec:
      raise errors.SourceScannerError(u'Invalid or missing volume scan node.')

    selected_vss_stores = []
    if not volume_scan_node.sub_nodes:
      self._ScanVolumeScanNode(
          scan_context, volume_scan_node, selected_vss_stores)

    else:
      # Some volumes contain other volume or file systems e.g. BitLocker ToGo
      # has an encrypted and unencrypted volume.
      for sub_scan_node in volume_scan_node.sub_nodes:
        self._ScanVolumeScanNode(
            scan_context, sub_scan_node, selected_vss_stores)

  def _ScanVolumeScanNode(
      self, scan_context, volume_scan_node, selected_vss_stores):
    """Scans an individual volume scan node for volume and file systems.

    Args:
      scan_context (dfvfs.SourceScannerContext): source scanner context.
      volume_scan_node (dfvfs.SourceScanNode): volume scan node.
      selected_vss_stores (list[str]): selected VSS store identifiers.

    Raises:
      SourceScannerError: if the format of or within the source
          is not supported or the the scan node is invalid.
    """
    if not volume_scan_node or not volume_scan_node.path_spec:
      raise errors.SourceScannerError(u'Invalid or missing volume scan node.')

    # Get the first node where where we need to decide what to process.
    scan_node = volume_scan_node
    while len(scan_node.sub_nodes) == 1:
      # Make sure that we prompt the user about VSS selection.
      if scan_node.type_indicator == dfvfs_definitions.TYPE_INDICATOR_VSHADOW:
        location = getattr(scan_node.path_spec, u'location', None)
        if location == u'/':
          break

      scan_node = scan_node.sub_nodes[0]

    # The source scanner found an encrypted volume and we need
    # a credential to unlock the volume.
    if scan_node.type_indicator in (
        dfvfs_definitions.ENCRYPTED_VOLUME_TYPE_INDICATORS):
      self._ScanVolumeScanNodeEncrypted(scan_context, scan_node)

    elif scan_node.type_indicator == dfvfs_definitions.TYPE_INDICATOR_VSHADOW:
      self._ScanVolumeScanNodeVSS(scan_node, selected_vss_stores)

    elif scan_node.type_indicator in (
        dfvfs_definitions.FILE_SYSTEM_TYPE_INDICATORS):
      if (not self._vss_only or not selected_vss_stores or
          self._PromptUserForVSSCurrentVolume()):
        self._source_path_specs.append(scan_node.path_spec)

  def _ScanVolumeScanNodeEncrypted(self, scan_context, volume_scan_node):
    """Scans an encrypted volume scan node for volume and file systems.

    Args:
      scan_context (dfvfs.SourceScannerContext): source scanner context.
      volume_scan_node (dfvfs.SourceScanNode): volume scan node.
    """
    result = not scan_context.IsLockedScanNode(volume_scan_node.path_spec)
    if not result:
      credentials = credentials_manager.CredentialsManager.GetCredentials(
          volume_scan_node.path_spec)

      result = False
      for credential_type, credential_data in self._credentials:
        if credential_type not in credentials.CREDENTIALS:
          continue

        result = self._source_scanner.Unlock(
            scan_context, volume_scan_node.path_spec, credential_type,
            credential_data)

        if result:
          self._AddCredentialConfiguration(
              volume_scan_node.path_spec, credential_type, credential_data)
          break

      if self._credentials and not result:
        self._output_writer.Write(
            u'[WARNING] Unable to unlock encrypted volume using the provided '
            u'credentials.\n\n')

      if not result:
        result = self._PromptUserForEncryptedVolumeCredential(
            scan_context, volume_scan_node, credentials)

    if result:
      self._source_scanner.Scan(
          scan_context, scan_path_spec=volume_scan_node.path_spec)
      self._ScanVolume(scan_context, volume_scan_node)

  def _ScanVolumeScanNodeVSS(self, volume_scan_node, selected_vss_stores):
    """Scans a VSS volume scan node for volume and file systems.

    Args:
      scan_context (dfvfs.SourceScannerContext): source scanner context.
      volume_scan_node (dfvfs.SourceScanNode): volume scan node.
      selected_vss_stores (list[str]): selected VSS store identifiers.

    Raises:
      SourceScannerError: if a VSS sub scan node cannot be retrieved.
    """
    if not self._process_vss:
      return

    # Do not scan inside individual VSS store scan nodes.
    location = getattr(volume_scan_node.path_spec, u'location', None)
    if location != u'/':
      return

    vss_store_identifiers = self._GetVSSStoreIdentifiers(
        volume_scan_node, vss_stores=self._vss_stores)

    selected_vss_stores.extend(vss_store_identifiers)

    # Process VSS stores starting with the most recent one.
    vss_store_identifiers.reverse()
    for vss_store_identifier in vss_store_identifiers:
      location = u'/vss{0:d}'.format(vss_store_identifier)
      sub_scan_node = volume_scan_node.GetSubNodeByLocation(location)
      if not sub_scan_node:
        logging.error(
            u'Scan node missing for VSS store identifier: {0:d}.'.format(
                vss_store_identifier))
        continue

      # We "optimize" here for user experience, ideally we would scan for
      # a file system instead of hard coding a TSK child path specification.
      path_spec = path_spec_factory.Factory.NewPathSpec(
          dfvfs_definitions.TYPE_INDICATOR_TSK, location=u'/',
          parent=sub_scan_node.path_spec)
      self._source_path_specs.append(path_spec)

  def AddCredentialOptions(self, argument_group):
    """Adds the credential options to the argument group.

    The credential options are use to unlock encrypted volumes.

    Args:
      argument_group (argparse._ArgumentGroup): argparse argument group.
    """
    argument_group.add_argument(
        u'--credential', action=u'append', default=[], type=str,
        dest=u'credentials', metavar=u'TYPE:DATA', help=(
            u'Define a credentials that can be used to unlock encrypted '
            u'volumes e.g. BitLocker. The credential is defined as type:data '
            u'e.g. "password:BDE-test". Supported credential types are: '
            u'{0:s}. Binary key data is expected to be passed in BASE-16 '
            u'encoding (hexadecimal). WARNING credentials passed via command '
            u'line arguments can end up in logs, so use this option with '
            u'care.').format(u', '.join(self._SUPPORTED_CREDENTIAL_TYPES)))

  def AddStorageMediaImageOptions(self, argument_group):
    """Adds the storage media image options to the argument group.

    Args:
      argument_group (argparse._ArgumentGroup): argparse argument group.
    """
    argument_group.add_argument(
        u'--partition', dest=u'partition', action=u'store', type=str,
        default=None, help=(
            u'Choose a partition number from a disk image. This partition '
            u'number should correspond to the partition number on the disk '
            u'image, starting from partition 1. All partitions can be '
            u'defined as: "all".'))

    argument_group.add_argument(
        u'--partitions', dest=u'partitions', action=u'store', type=str,
        default=None, help=(
            u'Define partitions that need to be processed. A range of '
            u'partitions can be defined as: "3..5". Multiple partitions can '
            u'be defined as: "1,3,5" (a list of comma separated values). '
            u'Ranges and lists can also be combined as: "1,3..5". The first '
            u'partition is 1. All partition can be defined as: "all".'))

    argument_group.add_argument(
        u'-o', u'--offset', dest=u'image_offset', action=u'store', default=None,
        type=int, help=(
            u'The offset of the volume within the storage media image in '
            u'number of sectors. A sector is {0:d} bytes in size by default '
            u'this can be overwritten with the --sector_size option.').format(
                self._DEFAULT_BYTES_PER_SECTOR))

    argument_group.add_argument(
        u'--ob', u'--offset_bytes', u'--offset_bytes',
        dest=u'image_offset_bytes', action=u'store', default=None, type=int,
        help=(
            u'The offset of the volume within the storage media image in '
            u'number of bytes.'))

    argument_group.add_argument(
        u'--sector_size', u'--sector-size', dest=u'bytes_per_sector',
        action=u'store', type=int, default=self._DEFAULT_BYTES_PER_SECTOR,
        help=(
            u'The number of bytes per sector, which is {0:d} by '
            u'default.').format(self._DEFAULT_BYTES_PER_SECTOR))

  def AddVSSProcessingOptions(self, argument_group):
    """Adds the VSS processing options to the argument group.

    Args:
      argument_group (argparse._ArgumentGroup): argparse argument group.
    """
    argument_group.add_argument(
        u'--no_vss', u'--no-vss', dest=u'no_vss', action=u'store_true',
        default=False, help=(
            u'Do not scan for Volume Shadow Snapshots (VSS). This means that '
            u'Volume Shadow Snapshots (VSS) are not processed.'))

    argument_group.add_argument(
        u'--vss_only', u'--vss-only', dest=u'vss_only', action=u'store_true',
        default=False, help=(
            u'Do not process the current volume if Volume Shadow Snapshots '
            u'(VSS) have been selected.'))

    argument_group.add_argument(
        u'--vss_stores', u'--vss-stores', dest=u'vss_stores', action=u'store',
        type=str, default=None, help=(
            u'Define Volume Shadow Snapshots (VSS) (or stores that need to be '
            u'processed. A range of stores can be defined as: "3..5". '
            u'Multiple stores can be defined as: "1,3,5" (a list of comma '
            u'separated values). Ranges and lists can also be combined as: '
            u'"1,3..5". The first store is 1. All stores can be defined as: '
            u'"all".'))

  def ScanSource(self):
    """Scans the source path for volume and file systems.

    This function sets the internal source path specification and source
    type values.

    Returns:
      dfvfs.SourceScannerContext: source scanner context.

    Raises:
      SourceScannerError: if the format of or within the source is
          not supported.
    """
    if (not self._source_path.startswith(u'\\\\.\\') and
        not os.path.exists(self._source_path)):
      raise errors.SourceScannerError(
          u'No such device, file or directory: {0:s}.'.format(
              self._source_path))

    scan_context = source_scanner.SourceScannerContext()
    scan_context.OpenSourcePath(self._source_path)

    try:
      self._source_scanner.Scan(scan_context)
    except (dfvfs_errors.BackEndError, ValueError) as exception:
      raise errors.SourceScannerError(
          u'Unable to scan source with error: {0:s}.'.format(exception))

    if scan_context.source_type not in (
        scan_context.SOURCE_TYPE_STORAGE_MEDIA_DEVICE,
        scan_context.SOURCE_TYPE_STORAGE_MEDIA_IMAGE):
      scan_node = scan_context.GetRootScanNode()
      self._source_path_specs.append(scan_node.path_spec)
      return scan_context

    # Get the first node where where we need to decide what to process.
    scan_node = scan_context.GetRootScanNode()
    while len(scan_node.sub_nodes) == 1:
      scan_node = scan_node.sub_nodes[0]

    # The source scanner found a partition table and we need to determine
    # which partition needs to be processed.
    if scan_node.type_indicator != (
        dfvfs_definitions.TYPE_INDICATOR_TSK_PARTITION):
      partition_identifiers = None

    else:
      partition_identifiers = self._GetTSKPartitionIdentifiers(
          scan_node, partition_offset=self._partition_offset,
          partitions=self._partitions)

    if not partition_identifiers:
      self._ScanVolume(scan_context, scan_node)

    else:
      for partition_identifier in partition_identifiers:
        location = u'/{0:s}'.format(partition_identifier)
        sub_scan_node = scan_node.GetSubNodeByLocation(location)
        self._ScanVolume(scan_context, sub_scan_node)

    if not self._source_path_specs:
      raise errors.SourceScannerError(
          u'No supported file system found in source.')

    return scan_context



# -*- coding: utf-8 -*-
"""The Android mmssms.db database event formatter."""

from plaso.formatters import interface
from plaso.formatters import manager


class AndroidSmsFormatter(interface.ConditionalEventFormatter):
  """Formatter for an Android SMS event."""

  DATA_TYPE = u'android:messaging:sms'

  FORMAT_STRING_PIECES = [
      u'Type: {sms_type}',
      u'Address: {address}',
      u'Status: {sms_read}',
      u'Message: {body}']

  FORMAT_STRING_SHORT_PIECES = [u'{body}']

  SOURCE_LONG = u'Android SMS messages'
  SOURCE_SHORT = u'SMS'


manager.FormattersManager.RegisterFormatter(AndroidSmsFormatter)



# -*- coding: utf-8 -*-
"""Tests for the SHA-256 hasher implementation."""

import unittest

from plaso.analyzers.hashers import sha256

from tests import test_lib as shared_test_lib
from tests.analyzers.hashers import test_lib


class SHA256Test(test_lib.HasherTestCase):
  """Tests the SHA-256 hasher."""

  @shared_test_lib.skipUnlessHasTestFile([u'empty_file'])
  def testFileHashMatchesEmptyFile(self):
    """Tests that hasher matches the hash of an empty file."""
    expected_sha256 = (
        u'e3b0c44298fc1c149afbf4c8996fb92427ae41e4649b934ca495991b7852b855')

    hasher = sha256.SHA256Hasher()
    self._AssertTestPathStringDigestMatch(
        hasher, [u'empty_file'], expected_sha256)

    hasher = sha256.SHA256Hasher()
    self._AssertTestPathBinaryDigestMatch(
        hasher, [u'empty_file'], expected_sha256.decode(u'hex'))

  @shared_test_lib.skipUnlessHasTestFile([u'ímynd.dd'])
  def testFileHashMatchesKnownFile(self):
    """Tests that hasher matches the hash of a known file."""
    expected_sha256 = (
        u'a9c0220b1dadd1812fc4bbe137f495d2c3b88c7b33b5a4de545d201fd31b3bc0')

    hasher = sha256.SHA256Hasher()
    self._AssertTestPathStringDigestMatch(
        hasher, [u'ímynd.dd'], expected_sha256)

    hasher = sha256.SHA256Hasher()
    self._AssertTestPathBinaryDigestMatch(
        hasher, [u'ímynd.dd'], expected_sha256.decode(u'hex'))


if __name__ == '__main__':
  unittest.main()



# -*- coding: utf-8 -*-
"""An output module that saves events to Elasticsearch."""

from collections import Counter
import logging

from dfvfs.serializer.json_serializer import JsonPathSpecSerializer

try:
  from elasticsearch import Elasticsearch
  from elasticsearch.exceptions import ConnectionError
except ImportError:
  Elasticsearch = None

from plaso.lib import errors
from plaso.lib import timelib
from plaso.output import interface
from plaso.output import manager

# Configure Elasticsearch logger
elastic_logger = logging.getLogger(u'elasticsearch.trace')
elastic_logger.setLevel(logging.WARNING)


class ElasticSearchHelper(object):
  """Elasticsearch helper class."""

  def __init__(
      self, output_mediator, host, port, flush_interval, index_name, mapping,
      doc_type, elastic_password=None, elastic_user=None):
    """Create a Elasticsearch helper.

    Args:
      output_mediator (OutputMediator): The output mediator object.
      host (str): IP address or hostname for the server.
      port (int): Port number for the server.
      flush_interval (int): How many events to queue before being indexed.
      index_name (str): Name of the Elasticsearch index.
      mapping (dict): Elasticsearch index configuration.
      doc_type (str): Elasticsearch document type name.
      elastic_passsword (Optional[str]): Elasticsearch password to authenticate
          with.
      elastic_user (Optional[str]): Elasticsearch username to authenticate with.
    """
    super(ElasticSearchHelper, self).__init__()

    elastic_hosts = [{u'host': host, u'port': port}]
    if elastic_user is None:
      self.client = Elasticsearch(elastic_hosts)
    else:
      self.client = Elasticsearch(
          elastic_hosts, http_auth=(elastic_user, elastic_password))

    self._output_mediator = output_mediator
    self._index = self._EnsureIndexExists(index_name, mapping)
    self._doc_type = doc_type
    self._flush_interval = flush_interval
    self._events = []
    self._counter = Counter()
    self._elastic_user = elastic_user
    self._elastic_password = elastic_password

  def AddEvent(self, event_object, force_flush=False):
    """Index event in Elasticsearch.

    Args:
      event_object (EventObject): the event object.
      force_flush (bool): Force bulk insert of events in the queue.
    """
    if event_object:
      self._events.append(
          {u'index': {u'_index': self._index, u'_type': self._doc_type}})
      self._events.append(self._GetSanitizedEventValues(event_object))
      self._counter[u'events'] += 1

    # Check if we need to flush the queued events
    if force_flush or self._counter[u'events'] % self._flush_interval == 0:
      self._FlushEventsToElasticSearch()

  def _EnsureIndexExists(self, index_name, mapping):
    """Create Elasticsearch index.

    Args:
      index_name: Name of the index.
      mapping: Mapping for the index

    Returns:
      Name of the index.
    """
    try:
      if not self.client.indices.exists(index_name):
        self.client.indices.create(
            index=index_name, body={u'mappings': mapping})
    except ConnectionError:
      raise RuntimeError(u'Unable to connect to Elasticsearch backend.')
    return index_name

  def _GetSanitizedEventValues(self, event_object):
    """Builds a dictionary from an event_object.

    The event object need to be sanitized to prevent certain values from causing
    problems when indexing with Elasticsearch. For example the value of
    the pathspec attribute is a nested dictionary which will cause problems for
    Elasticsearch automatic indexing.

    Args:
      event_object: the event object (instance of EventObject).

    Returns:
      Dictionary with sanitized event object values.
    """
    event_values = {}
    for attribute_name, attribute_value in event_object.GetAttributes():
      # Ignore the regvalue attribute as it cause issues when indexing
      if attribute_name == u'regvalue':
        continue

      if attribute_name == u'pathspec':
        try:
          attribute_value = JsonPathSpecSerializer.WriteSerialized(
              attribute_value)
        except TypeError:
          continue
      event_values[attribute_name] = attribute_value

    # Add string representation of the timestamp
    attribute_value = timelib.Timestamp.RoundToSeconds(event_object.timestamp)
    attribute_value = timelib.Timestamp.CopyToIsoFormat(
        attribute_value, timezone=self._output_mediator.timezone)
    event_values[u'datetime'] = attribute_value

    message, _ = self._output_mediator.GetFormattedMessages(event_object)
    if message is None:
      raise errors.NoFormatterFound(
          u'Unable to find event formatter for: {0:s}.'.format(
              getattr(event_object, u'data_type', u'UNKNOWN')))
    event_values[u'message'] = message

    # Tags needs to be a list for Elasticsearch to index correctly.
    try:
      labels = list(event_values[u'tag'].labels)
    except (KeyError, AttributeError):
      labels = []
    event_values[u'tag'] = labels

    source_short, source = self._output_mediator.GetFormattedSources(
        event_object)
    if source is None or source_short is None:
      raise errors.NoFormatterFound(
          u'Unable to find event formatter for: {0:s}.'.format(
              getattr(event_object, u'data_type', u'UNKNOWN')))
    event_values[u'source_short'] = source_short
    event_values[u'source_long'] = source

    return event_values

  def _FlushEventsToElasticSearch(self):
    """Insert events in bulk to Elasticsearch."""
    try:
      self.client.bulk(
          index=self._index, doc_type=self._doc_type, body=self._events)
    except ValueError as e:
      # Ignore problematic events
      logging.warning(u'{0:s}'.format(e))

    # Clear the events list
    self._events = []
    logging.info(u'{0:d} events added'.format(self._counter[u'events']))


class ElasticSearchOutputModule(interface.OutputModule):
  """Output module for Elasticsearch."""

  NAME = u'elastic'
  DESCRIPTION = u'Saves the events into an Elasticsearch database.'

  # Strings longer than this will not be analyzed by elasticsearch.
  _ELASTIC_ANALYZER_STRING_LIMIT = 10922

  def __init__(self, output_mediator):
    """Initializes the output module object.

    Args:
      output_mediator: The output mediator object (instance of OutputMediator).
    """
    super(ElasticSearchOutputModule, self).__init__(output_mediator)
    self._doc_type = None
    self._elastic = None
    self._elastic_password = None
    self._elastic_user = None
    self._flush_interval = None
    self._host = None
    self._index_name = None
    self._mapping = None
    self._output_mediator = output_mediator
    self._port = None
    self._raw_fields = False

  def Close(self):
    """Close connection to the Elasticsearch database.

    Sends any remaining buffered events for indexing.
    """
    self._elastic.AddEvent(event_object=None, force_flush=True)

  def SetServerInformation(self, server, port):
    """Set the Elasticsearch server information.

    Args:
      server (str): IP address or hostname of the server.
      port (int): Port number of the server.
    """
    self._host = server
    self._port = port
    logging.info(u'Server address: {0:s}'.format(self._host))
    logging.info(u'Server port: {0:d}'.format(self._port))

  def SetFlushInterval(self, flush_interval):
    """Set the flush interval.

    Args:
      flush_interval (int): Number of events to buffer before bulk insert.
    """
    self._flush_interval = flush_interval
    logging.info(u'Flush interval: {0:d}'.format(self._flush_interval))

  def SetIndexName(self, index_name):
    """Set the index name.

    Args:
      index_name: the index name.
    """
    self._index_name = index_name
    logging.info(u'Index name: {0:s}'.format(self._index_name))

  def SetDocType(self, doc_type):
    """Set the port.

    Args:
      doc_type (str): The document type to use when indexing.
    """
    self._doc_type = doc_type
    logging.info(u'Document type: {0:s}'.format(self._doc_type))

  def SetRawFields(self, raw_fields):
    """Set raw (not analyzed) fields.

    This is used for sorting and aggregations in Elasticsearch.
    https://www.elastic.co/guide/en/elasticsearch/guide/current/
    multi-fields.html

    Args:
      raw_fields (bool): Add not-analyzed index for string fields.
    """
    self._raw_fields = raw_fields
    logging.info(u'Add non analyzed string fields: {0!s}'.format(
        self._raw_fields))

  def SetElasticUser(self, elastic_user):
    """Set the Elastic username.

    Args:
      elastic_user (str): Elastic user to authenticate with.
    """
    self._elastic_user = elastic_user
    logging.info(u'Elastic user: {0:s}'.format(self._elastic_user))

  def SetElasticPassword(self, elastic_password):
    """Set the Elastic password.

    Args:
      elastic_password (str): Elastic password to authenticate with.
    """
    self._elastic_password = elastic_password
    logging.info(u'Elastic password: {0:s}'.format(u'****'))

  def WriteEventBody(self, event_object):
    """Writes the body of an event object to the output.

    Args:
      event_object: the event object (instance of EventObject).
    """
    self._elastic.AddEvent(event_object)

  def WriteHeader(self):
    """Setup the Elasticsearch index."""
    if not self._mapping:
      self._mapping = {}

    if self._raw_fields:
      if self._doc_type not in self._mapping:
        self._mapping[self._doc_type] = {}

      _raw_field_mapping = [{
          u'strings': {
              u'match_mapping_type': u'string',
              u'mapping': {
                  u'fields': {
                      u'raw': {
                          u'type': u'string',
                          u'index': u'not_analyzed',
                          u'ignore_above': self._ELASTIC_ANALYZER_STRING_LIMIT
                      }
                  }
              }
          }
      }]
      self._mapping[self._doc_type][u'dynamic_templates'] = _raw_field_mapping

    self._elastic = ElasticSearchHelper(
        self._output_mediator, self._host, self._port, self._flush_interval,
        self._index_name, self._mapping, self._doc_type,
        elastic_password=self._elastic_password,
        elastic_user=self._elastic_user)
    logging.info(u'Adding events to Elasticsearch..')


manager.OutputManager.RegisterOutput(
    ElasticSearchOutputModule, disabled=Elasticsearch is None)



# -*- coding: utf-8 -*-
"""The data location CLI arguments helper."""

from __future__ import unicode_literals

import os
import sys

from plaso.cli import tools
from plaso.cli.helpers import interface
from plaso.cli.helpers import manager
from plaso.lib import errors


class DataLocationArgumentsHelper(interface.ArgumentsHelper):
  """Data location CLI arguments helper."""

  NAME = 'data_location'
  DESCRIPTION = 'Data location command line arguments.'

  # Preserve the absolute path value of __file__ in case it is changed
  # at run-time.
  _PATH = os.path.abspath(__file__)

  @classmethod
  def AddArguments(cls, argument_group):
    """Adds command line arguments to an argument group.

    This function takes an argument parser or an argument group object and adds
    to it all the command line arguments this helper supports.

    Args:
      argument_group (argparse._ArgumentGroup|argparse.ArgumentParser):
          argparse group.
    """
    argument_group.add_argument(
        '--data', action='store', dest='data_location', type=str,
        metavar='PATH', default=None, help='the location of the data files.')

  @classmethod
  def ParseOptions(cls, options, configuration_object):
    """Parses and validates options.

    Args:
      options (argparse.Namespace): parser options.
      configuration_object (CLITool): object to be configured by the argument
          helper.

    Raises:
      BadConfigObject: when the configuration object is of the wrong type.
    """
    if not isinstance(configuration_object, tools.CLITool):
      raise errors.BadConfigObject(
          'Configuration object is not an instance of CLITool')

    data_location = cls._ParseStringOption(options, 'data_location')
    if not data_location:
      # Determine the source root path, which is 3 directories up from
      # the location of the script.
      data_location = os.path.dirname(cls._PATH)
      data_location = os.path.dirname(data_location)
      data_location = os.path.dirname(data_location)
      data_location = os.path.dirname(data_location)

      # There are multiple options to run a tool e.g. running from source or
      # from an egg file.
      data_location_egg = os.path.join(data_location, 'share', 'plaso')
      data_location_source = os.path.join(data_location, 'data')

      if os.path.exists(data_location_egg):
        data_location = data_location_egg
      elif os.path.exists(data_location_source):
        data_location = data_location_source

      if not os.path.exists(data_location):
        data_location = os.path.join(sys.prefix, 'share', 'plaso')
      if not os.path.exists(data_location):
        data_location = os.path.join(sys.prefix, 'local', 'share', 'plaso')

      if sys.prefix != '/usr':
        if not os.path.exists(data_location):
          data_location = os.path.join('/usr', 'share', 'plaso')
        if not os.path.exists(data_location):
          data_location = os.path.join('/usr', 'local', 'share', 'plaso')

      if not os.path.exists(data_location):
        data_location = None

    if not data_location:
      raise errors.BadConfigOption(
          'Unable to determine location of data files.')

    setattr(configuration_object, '_data_location', data_location)


manager.ArgumentHelperManager.RegisterHelper(DataLocationArgumentsHelper)



#!/usr/bin/python
# -*- coding: utf-8 -*-
"""Tests for the event formatters manager."""

import unittest

from plaso.formatters import manager
from plaso.formatters import mediator
from plaso.formatters import winreg  # pylint: disable=unused-import

from tests import test_lib as shared_test_lib
from tests.containers import test_lib as containers_test_lib
from tests.formatters import test_lib


class FormattersManagerTest(shared_test_lib.BaseTestCase):
  """Tests for the event formatters manager."""

  def testFormatterRegistration(self):
    """Tests the RegisterFormatter and DeregisterFormatter functions."""
    # pylint: disable=protected-access
    number_of_formatters = len(manager.FormattersManager._formatter_classes)

    manager.FormattersManager.RegisterFormatter(test_lib.TestEventFormatter)
    self.assertEqual(
        len(manager.FormattersManager._formatter_classes),
        number_of_formatters + 1)

    with self.assertRaises(KeyError):
      manager.FormattersManager.RegisterFormatter(test_lib.TestEventFormatter)

    manager.FormattersManager.DeregisterFormatter(test_lib.TestEventFormatter)
    self.assertEqual(
        len(manager.FormattersManager._formatter_classes),
        number_of_formatters)

  def testMessageStrings(self):
    """Tests the GetMessageStrings and GetSourceStrings functions."""
    manager.FormattersManager.RegisterFormatter(test_lib.TestEventFormatter)

    formatter_mediator = mediator.FormatterMediator()

    message_strings = []
    text_message = None
    text_message_short = None

    test_events = containers_test_lib.CreateTestEvents()
    for event in test_events:
      message, message_short = manager.FormattersManager.GetMessageStrings(
          formatter_mediator, event)
      source_short, source_long = manager.FormattersManager.GetSourceStrings(
          event)

      if source_short == u'LOG' and not text_message:
        text_message = message
        text_message_short = message_short

      csv_message_strings = u'{0:d},{1:s},{2:s},{3:s}'.format(
          event.timestamp, source_short, source_long, message)
      message_strings.append(csv_message_strings)

    self.assertIn((
        u'1334961526929596,REG,UNKNOWN,[MY AutoRun key] '
        u'Value: c:/Temp/evil.exe'), message_strings)

    self.assertIn(
        (u'1334966206929596,REG,UNKNOWN,'
         u'[HKEY_CURRENT_USER\\Secret\\EvilEmpire\\Malicious_key] '
         u'Value: send all the exes to the other world'), message_strings)
    self.assertIn(
        (u'1334940286000000,REG,UNKNOWN,'
         u'[HKEY_CURRENT_USER\\Windows\\Normal] '
         u'Value: run all the benign stuff'), message_strings)
    self.assertIn(
        (u'1335781787929596,FILE,Weird Log File,This log line reads '
         u'ohh so much.'), message_strings)
    self.assertIn(
        (u'1335781787929596,FILE,Weird Log File,Nothing of interest '
         u'here, move on.'), message_strings)
    self.assertIn(
        (u'1335791207939596,FILE,Weird Log File,Mr. Evil just logged '
         u'into the machine and got root.'), message_strings)

    expected_text_message = (
        u'This is a line by someone not reading the log line properly. And '
        u'since this log line exceeds the accepted 80 chars it will be '
        u'shortened.')
    self.assertEqual(text_message, expected_text_message)

    expected_text_message_short = (
        u'This is a line by someone not reading the log line properly. '
        u'And since this l...')
    self.assertEqual(text_message_short, expected_text_message_short)

    manager.FormattersManager.DeregisterFormatter(test_lib.TestEventFormatter)


if __name__ == '__main__':
  unittest.main()



# -*- coding: utf-8 -*-
"""The bencode parser event formatters."""

from plaso.formatters import interface
from plaso.formatters import manager


class TransmissionEventFormatter(interface.ConditionalEventFormatter):
  """Formatter for a Transmission active torrents event."""

  DATA_TYPE = u'p2p:bittorrent:transmission'

  SOURCE_LONG = u'Transmission Active Torrents'
  SOURCE_SHORT = u'TORRENT'

  FORMAT_STRING_SEPARATOR = u'; '

  FORMAT_STRING_PIECES = [
      u'Saved to {destination}',
      u'Minutes seeded: {seedtime}']


class UTorrentEventFormatter(interface.ConditionalEventFormatter):
  """Formatter for a BitTorrent uTorrent active torrents event."""

  DATA_TYPE = u'p2p:bittorrent:utorrent'

  SOURCE_LONG = u'uTorrent Active Torrents'
  SOURCE_SHORT = u'TORRENT'

  FORMAT_STRING_SEPARATOR = u'; '

  FORMAT_STRING_PIECES = [
      u'Torrent {caption}',
      u'Saved to {path}',
      u'Minutes seeded: {seedtime}']


manager.FormattersManager.RegisterFormatters([
    TransmissionEventFormatter, UTorrentEventFormatter])



#!/usr/bin/python
# -*- coding: utf-8 -*-
"""Tests for the filter file CLI arguments helper."""

import argparse
import unittest

from plaso.cli import tools
from plaso.cli.helpers import filter_file
from plaso.lib import errors

from tests.cli import test_lib as cli_test_lib


class FilterFileArgumentsHelperTest(cli_test_lib.CLIToolTestCase):
  """Tests for the filter file CLI arguments helper."""

  # pylint: disable=protected-access

  _EXPECTED_OUTPUT = u'\n'.join([
      u'usage: cli_helper.py [-f FILE_FILTER]',
      u'',
      u'Test argument parser.',
      u'',
      u'optional arguments:',
      u'  -f FILE_FILTER, --file_filter FILE_FILTER, --file-filter FILE_FILTER',
      (u'                        List of files to include for targeted '
       u'collection of'),
      (u'                        files to parse, one line per file path, '
       u'setup is'),
      (u'                        /path|file - where each element can contain '
       u'either a'),
      (u'                        variable set in the preprocessing stage or '
       u'a regular'),
      u'                        expression.',
      u''])

  def testAddArguments(self):
    """Tests the AddArguments function."""
    argument_parser = argparse.ArgumentParser(
        prog=u'cli_helper.py', description=u'Test argument parser.',
        add_help=False,
        formatter_class=cli_test_lib.SortedArgumentsHelpFormatter)

    filter_file.FilterFileArgumentsHelper.AddArguments(argument_parser)

    output = self._RunArgparseFormatHelp(argument_parser)
    self.assertEqual(output, self._EXPECTED_OUTPUT)

  def testParseOptions(self):
    """Tests the ParseOptions function."""
    options = cli_test_lib.TestOptions()
    options.file_filter = self._GetTestFilePath([u'testdir', u'filter2.txt'])

    test_tool = tools.CLITool()
    filter_file.FilterFileArgumentsHelper.ParseOptions(options, test_tool)

    self.assertEqual(test_tool._filter_file, options.file_filter)

    with self.assertRaises(errors.BadConfigObject):
      filter_file.FilterFileArgumentsHelper.ParseOptions(options, None)

    # TODO: improve test coverage.


if __name__ == '__main__':
  unittest.main()



#!/usr/bin/python
# -*- coding: utf-8 -*-
"""Tests for the artifacts attribute containers."""

import unittest

from plaso.containers import artifacts

from tests import test_lib as shared_test_lib


class EnvironmentVariableArtifactTest(shared_test_lib.BaseTestCase):
  """Tests for the environment variable aritifact."""

  # TODO: replace by GetAttributeNames test
  def testCopyToDict(self):
    """Tests the CopyToDict function."""
    attribute_container = artifacts.EnvironmentVariableArtifact(
        case_sensitive=False, name=u'SystemRoot', value=u'C:\\Windows')

    self.assertEqual(attribute_container.name, u'SystemRoot')

    expected_dict = {
        u'case_sensitive': False,
        u'name': u'SystemRoot',
        u'value': u'C:\\Windows'}

    test_dict = attribute_container.CopyToDict()

    self.assertEqual(test_dict, expected_dict)


class HostnameArtifactTest(shared_test_lib.BaseTestCase):
  """Tests for the hostname aritifact."""

  # TODO: replace by GetAttributeNames test
  def testCopyToDict(self):
    """Tests the CopyToDict function."""
    attribute_container = artifacts.HostnameArtifact(name=u'mydomain.com')

    self.assertEqual(attribute_container.name, u'mydomain.com')

    expected_dict = {
        u'name': u'mydomain.com',
        u'schema': u'DNS'}

    test_dict = attribute_container.CopyToDict()

    self.assertEqual(test_dict, expected_dict)


class SystemConfigurationArtifactTest(shared_test_lib.BaseTestCase):
  """Tests for the system configuration aritifact."""

  # TODO: replace by GetAttributeNames test
  def testCopyToDict(self):
    """Tests the CopyToDict function."""
    attribute_container = artifacts.SystemConfigurationArtifact(
        code_page=u'cp1252', time_zone=u'UTC')

    self.assertEqual(attribute_container.time_zone, u'UTC')

    expected_dict = {
        u'code_page': u'cp1252',
        u'time_zone': u'UTC',
        u'user_accounts': []}

    test_dict = attribute_container.CopyToDict()

    self.assertEqual(test_dict, expected_dict)


class UserAccountArtifactTest(shared_test_lib.BaseTestCase):
  """Tests for the user account artifact."""

  # TODO: replace by GetAttributeNames test
  def testCopyToDict(self):
    """Tests the CopyToDict function."""
    attribute_container = artifacts.UserAccountArtifact(
        full_name=u'Full Name', group_identifier=1001, identifier=1000,
        user_directory=u'/home/username', username=u'username')

    self.assertEqual(attribute_container.username, u'username')

    expected_dict = {
        u'full_name': u'Full Name',
        u'group_identifier': 1001,
        u'identifier': 1000,
        u'user_directory': u'/home/username',
        u'username': u'username'}

    test_dict = attribute_container.CopyToDict()

    self.assertEqual(test_dict, expected_dict)


if __name__ == '__main__':
  unittest.main()



#!/usr/bin/python
# -*- coding: utf-8 -*-
"""Tests for the Timesketch output module CLI arguments helper."""

import argparse
import unittest

from plaso.cli.helpers import timesketch_output
from plaso.lib import errors
from plaso.output import timesketch_out

from tests.cli import test_lib as cli_test_lib
from tests.cli.helpers import test_lib


class MockTimesketchOutputModule(timesketch_out.TimesketchOutputModule):
  """Mock Timesketch output module for testing."""

  def __init__(self, output_mediator):
    """Initializes an output module object.

    Args:
      output_mediator (OutputMediator): mediates interactions between output
          modules and other components, such as storage and dfvfs.
    """
    # We deliberately don't call super here to prevent the call to
    # timesketch.create_app() raising an exception.
    # pylint: disable=super-init-not-called
    pass


class TimesketchOutputArgumentsHelperTest(
    test_lib.OutputModuleArgumentsHelperTest):
  """Tests the Timesketch output module CLI arguments helper."""

  _EXPECTED_OUTPUT = u'\n'.join([
      u'usage: cli_helper.py [--name TIMELINE_NAME] [--index INDEX]',
      (u'                     [--flush_interval FLUSH_INTERVAL] '
       u'[--doc_type DOC_TYPE]'),
      u'                     [--username USERNAME]',
      u'',
      u'Test argument parser.',
      u'',
      u'optional arguments:',
      u'  --doc_type DOC_TYPE   Name of the document type that will be used in',
      u'                        ElasticSearch.',
      u'  --flush_interval FLUSH_INTERVAL, --flush-interval FLUSH_INTERVAL',
      (u'                        The number of events to queue up before sent '
       u'in bulk'),
      u'                        to Elasticsearch.',
      (u'  --index INDEX         The name of the Elasticsearch index. Default: '
       u'Generate'),
      u'                        a random UUID',
      (u'  --name TIMELINE_NAME, --timeline_name TIMELINE_NAME, '
       u'--timeline-name TIMELINE_NAME'),
      (u'                        The name of the timeline in Timesketch. '
       u'Default:'),
      u'                        hostname if present in the storage file. If no',
      u'                        hostname is found then manual input is used.',
      (u'  --username USERNAME   Username of a Timesketch user that will own '
       u'the'),
      u'                        timeline.',
      u''])

  def testAddArguments(self):
    """Tests the AddArguments function."""
    argument_parser = argparse.ArgumentParser(
        prog=u'cli_helper.py',
        description=u'Test argument parser.', add_help=False,
        formatter_class=cli_test_lib.SortedArgumentsHelpFormatter)

    timesketch_output.TimesketchOutputArgumentsHelper.AddArguments(
        argument_parser)

    output = self._RunArgparseFormatHelp(argument_parser)
    self.assertEqual(output, self._EXPECTED_OUTPUT)

  def testParseOptions(self):
    """Tests the ParseOptions function."""
    options = cli_test_lib.TestOptions()

    output_mediator = self._CreateOutputMediator()
    output_module = MockTimesketchOutputModule(output_mediator)
    timesketch_output.TimesketchOutputArgumentsHelper.ParseOptions(
        options, output_module)

    with self.assertRaises(errors.BadConfigObject):
      timesketch_output.TimesketchOutputArgumentsHelper.ParseOptions(
          options, None)


if __name__ == '__main__':
  unittest.main()
